# -*- coding: utf-8 -*-
import os
import pickle
import time
import traceback
from typing import Union

import happybase
from logzero import logger
from scrapy.http import Request
from scrapy.http import TextResponse, Response

import XX.Encrypt.EncryptHelper as Enc
from XX.Date.DatetimeHelper import get_today, get_this_month
from XX.File.FileHelper import FileHelper as Fh
from XX.HTML.UrlHelper import UrlHelper as Uh

process_request_type = Union[None, Response, Request]
process_response_type = Union[Response, Request]


# 文件缓存中间件
class CacheFileRequest(object):
    @staticmethod
    def get_cache_file(*args, **kwargs):
        spider = kwargs.get("spider")
        url = kwargs.get("url")
        request = kwargs.get("request", {})
        method = "get"
        if request:
            method = request.method
            body = request.body.decode("utf-8")
            # print("POST 缓存")
        else:
            # print("----No request")
            pass
        root_path_cache = kwargs.get("root_path_cache", "./")
        spider = spider if spider else Uh.get_domain(url).replace(".", "_")
        if method.lower() == "post":
            return root_path_cache + spider + os.sep + Fh.get_md5_name(url + "_" + body) + ".cache"
        else:
            return root_path_cache + spider + os.sep + Fh.get_md5_name(url) + ".cache"

    @classmethod
    def from_crawler(cls, crawler):

        settings = crawler.settings
        cls.cache_file_path_func = settings.get("FUN_CACHE_FILE_PATH", cls.get_cache_file)
        cls.cache_file_exclude = settings.get("CACHE_FILE_EXCLUDE")
        cls.cache_file_ts = settings.get("CACHE_FILE_TS", None)
        cls.cache_file_exclude_ts = settings.get("CACHE_FILE_EXCLUDE_TS", cls.cache_file_ts)
        cls.settings = settings
        return cls()

    def process_request(self, request, spider) -> process_request_type:
        cache_file_path = self.cache_file_path_func(
            url=str(request.url), spider=str(spider.name),
            root_path_cache=self.settings.get("ROOT_PATH_CACHE"),
            request=request)

        # 是否应该读缓存
        def should_read_cache(request, spider) -> bool:
            # 存在缓存文件
            exists_file = Fh.is_file_exit(cache_file_path)
            # 是排除URL
            exclude = 1 if self.cache_file_exclude and request.url.find(self.cache_file_exclude) >= 0 else 0
            if exists_file:
                if exclude:
                    if self.cache_file_exclude_ts >= 0:

                        if time.time() - Fh.get_update_ts(cache_file_path) < self.cache_file_exclude_ts:
                            return True
                        else:
                            logger.debug(
                                f"缓存文件过期。不读取。{cache_file_path}{Fh.get_update_ts(cache_file_path)}  {request.url}")
                            return False
                    # 排除URL无过期时间,就是排除URL都不读缓存
                    else:
                        logger.debug(f"排除的，没设置过期时间，全不读缓存。{request.url}")
                        return False
                else:
                    # 全部时间
                    if self.cache_file_ts:
                        # 全部URL 没有过期
                        if time.time() - Fh.get_update_ts(cache_file_path) < self.cache_file_ts:
                            return True
                        else:
                            logger.debug(f"== OUT TIME，不读缓存。{request.url}")
                            return False
                    else:
                        return True
            else:
                # logger.debug(f"缓存文件不存在。 {request.url}")
                return False

        if should_read_cache(request, spider):
            try:
                logger.info(f"===Read cache===  {cache_file_path.split(os.sep)[-1]}   {request.url}")
                return pickle.load(open(cache_file_path, "rb"))
            except Exception as e:
                logger.info(f"== Can't Read cache ===  Reason is {e}")
                Fh.remove_file(cache_file_path)
                traceback.print_exc()
        else:
            # 不读缓存
            # logger.info("===Should Not Read Cache===   " + request.url)
            pass

    # 写文件缓存
    def process_response(self, request, response, spider) -> process_response_type:
        cache_file_path = self.cache_file_path_func(
            url=str(request.url), spider=str(spider.name),
            root_path_cache=self.settings.get("ROOT_PATH_CACHE"),
            request=request)

        # 是否应该重写缓存
        def should_write_cache(request, spider) -> bool:
            exists_file = Fh.is_file_exit(cache_file_path)
            exclude = 1 if self.cache_file_exclude and request.url.find(self.cache_file_exclude) >= 0 else 0
            if exists_file:
                if exclude:
                    if self.cache_file_exclude_ts:
                        if time.time() - Fh.get_update_ts(cache_file_path) > self.cache_file_exclude_ts:
                            logger.debug(
                                f"排除缓存文件过期。需要重写。{cache_file_path}{Fh.get_update_ts(cache_file_path)}  {request.url}")
                            return True
                        else:
                            logger.debug(
                                f"排除的URL没过期，不重写。过了{int(time.time() - Fh.get_update_ts(cache_file_path))} 秒？ {cache_file_path}")
                            return False
                    else:
                        logger.debug(f"排除URL没设置过期时间 都要重写。 {cache_file_path}")
                        return True
                else:
                    # logger.debug(f"不是排除的URL不重写缓存。 {cache_file_path}")
                    return False
            else:
                logger.debug(f"缓存文件不存在。写缓存。 {request.url}")
                return True

        if response.status < 300:
            Fh.mkdir(Fh.get_file_path_and_name(cache_file_path)[0])
            try:
                if should_write_cache(request, spider):
                    if hasattr(response, "certificate"):
                        del response.certificate
                    res = Fh.remove_file(cache_file_path)
                    pickle.dump(response, open(cache_file_path, "wb"))
                    logger.info(f"===Write cache===      {cache_file_path}   {request.url}")
                # else:
                #     logger.info(f"===Cache Exists===      {cache_file_path}   {request.url}")
            except Exception as e:
                logger.info(f"== Can't Write cache 2 file==   Reason is {e}")
                Fh.remove_file(cache_file_path)
                # traceback.print_exc()
        else:
            logger.info(f"===Status Code ERROR ===   Code is {response.status} " + response.url)
        return response


# 文件按天缓存中间件
class CacheFilePerDayRequest(object):
    @staticmethod
    def get_cache_file(*args, **kwargs):
        spider = kwargs.get("spider")
        url = kwargs.get("url")
        request = kwargs.get("request", {})
        method = "get"
        if request:
            method = request.method
            body = request.body.decode("utf-8")
            # print("POST 缓存")
        else:
            # print("----No request")
            pass
        root_path_cache = kwargs.get("root_path_cache", "./")
        spider = spider if spider else Uh.get_domain(url).replace(".", "_")
        if method.lower() == "post":
            return root_path_cache + spider + os.sep + get_today().replace("-", '_') + os.sep + Fh.get_md5_name(url + "_" + body) + ".cache"
        else:
            return root_path_cache + spider + os.sep + get_today().replace("-", '_') + os.sep + Fh.get_md5_name(url) + ".cache"

    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.cache_file_path_func = settings.get("FUN_CACHE_FILE_PATH", cls.get_cache_file)
        cls.cache_file_exclude = settings.get("CACHE_FILE_EXCLUDE")
        cls.cache_file_ts = settings.get("CACHE_FILE_TS", None)
        cls.cache_file_exclude_ts = settings.get("CACHE_FILE_EXCLUDE_TS", cls.cache_file_ts)
        cls.settings = settings
        return cls()

    def process_request(self, request, spider) -> process_request_type:
        cache_file_path = self.cache_file_path_func(
            url=str(request.url), spider=str(spider.name),
            root_path_cache=self.settings.get("ROOT_PATH_CACHE"),
            request=request)

        # 是否应该读缓存
        def should_read_cache(request, spider) -> bool:
            # 存在缓存文件
            exists_file = Fh.is_file_exit(cache_file_path)
            # 是排除URL
            exclude = 1 if self.cache_file_exclude and request.url.find(self.cache_file_exclude) >= 0 else 0
            if exists_file:
                if exclude:
                    if self.cache_file_exclude_ts >= 0:
                        if time.time() - Fh.get_update_ts(cache_file_path) < self.cache_file_exclude_ts:
                            return True
                        else:
                            logger.debug(
                                f"缓存文件过期。不读取。{cache_file_path}{Fh.get_update_ts(cache_file_path)}  {request.url}")
                            return False
                    # 排除URL无过期时间,就是排除URL都不读缓存
                    else:
                        logger.debug(f"排除的，没设置过期时间，全不读缓存。{request.url}")
                        return False
                else:
                    # 全部时间
                    if self.cache_file_ts:
                        # 全部URL 没有过期
                        if time.time() - Fh.get_update_ts(cache_file_path) < self.cache_file_ts:
                            return True
                        else:
                            logger.debug(f"== OUT TIME，不读缓存。{request.url}")
                            return False
                    else:
                        return True
            else:
                # logger.debug(f"缓存文件不存在。 {request.url}")
                return False

        if should_read_cache(request, spider):
            try:
                logger.info(f"===Read cache===  {cache_file_path.split(os.sep)[-1]}   {request.url}")
                return pickle.load(open(cache_file_path, "rb"))
            except Exception as e:
                logger.info(f"== Can't Read cache ===  Reason is {e}")
                Fh.remove_file(cache_file_path)
                traceback.print_exc()
        else:
            # 不读缓存
            logger.info("===Shouldn't Read Cache===   " + request.url)
            pass

    # 写文件缓存
    def process_response(self, request, response, spider) -> process_response_type:
        cache_file_path = self.cache_file_path_func(
            url=str(request.url), spider=str(spider.name),
            root_path_cache=self.settings.get("ROOT_PATH_CACHE"),
            request=request)

        # 是否应该重写缓存
        def should_write_cache(request, spider) -> bool:
            exists_file = Fh.is_file_exit(cache_file_path)
            exclude = 1 if self.cache_file_exclude and request.url.find(self.cache_file_exclude) >= 0 else 0
            if exists_file:
                if exclude:
                    if self.cache_file_exclude_ts:
                        if time.time() - Fh.get_update_ts(cache_file_path) > self.cache_file_exclude_ts:
                            logger.debug(
                                f"排除缓存文件过期。需要重写。{cache_file_path}{Fh.get_update_ts(cache_file_path)}  {request.url}")
                            return True
                        else:
                            logger.debug(
                                f"排除的URL没过期，不重写。过了{int(time.time() - Fh.get_update_ts(cache_file_path))} 秒？ {cache_file_path}")
                            return False
                    else:
                        logger.debug(f"排除URL没设置过期时间 都要重写。 {cache_file_path}")
                        return True
                else:
                    # logger.debug(f"不是排除的URL不重写缓存。 {cache_file_path}")
                    return False
            else:
                logger.debug(f"缓存文件不存在。写缓存。 {request.url}")
                return True

        if response.status < 300:
            Fh.mkdir(Fh.get_file_path_and_name(cache_file_path)[0])
            try:
                if should_write_cache(request, spider):
                    if hasattr(response, "certificate"):
                        del response.certificate
                    Fh.remove_file(cache_file_path)
                    pickle.dump(response, open(cache_file_path, "wb"))
                    logger.info(f"===Write cache===      {cache_file_path}   {request.url}")
                # else:
                #     logger.info(f"===Cache Exists===      {cache_file_path}   {request.url}")
            except Exception as e:
                logger.info(f"== Can't Write cache 2 file==   Reason is {e}")
                Fh.remove_file(cache_file_path)
                # traceback.print_exc()
        else:
            logger.info(f"===Status Code ERROR ===   Code is {response.status} " + response.url)
        return response


# 文件按月缓存中间件
class CacheFilePerMonthRequest(object):
    @staticmethod
    def get_cache_file(*args, **kwargs):
        spider = kwargs.get("spider")
        url = kwargs.get("url")
        request = kwargs.get("request", {})
        method = "get"
        body = ''
        if request:
            method = request.method
            body = request.body.decode("utf-8")
            # print("POST 缓存")
        else:
            # print("----No request")
            pass
        root_path_cache = kwargs.get("root_path_cache", "./")
        spider = spider if spider else Uh.get_domain(url).replace(".", "_")
        if method.lower() == "post":
            return root_path_cache + spider + os.sep + get_this_month().replace("-", '_') + os.sep + Fh.get_md5_name(url + "_" + body) + ".cache"
        else:
            return root_path_cache + spider + os.sep + get_this_month().replace("-", '_') + os.sep + Fh.get_md5_name(url) + ".cache"

    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.cache_file_path_func = settings.get("FUN_CACHE_FILE_PATH", cls.get_cache_file)
        cls.cache_file_exclude = settings.get("CACHE_FILE_EXCLUDE")
        cls.cache_file_ts = settings.get("CACHE_FILE_TS", None)
        cls.cache_file_exclude_ts = settings.get("CACHE_FILE_EXCLUDE_TS", cls.cache_file_ts)
        cls.settings = settings
        return cls()

    def process_request(self, request, spider) -> process_request_type:
        cache_file_path = self.cache_file_path_func(
            url=str(request.url), spider=str(spider.name),
            root_path_cache=self.settings.get("ROOT_PATH_CACHE"),
            request=request)

        # 是否应该读缓存
        def should_read_cache(request, spider) -> bool:
            # 存在缓存文件
            exists_file = Fh.is_file_exit(cache_file_path)
            # 是排除URL
            exclude = 1 if self.cache_file_exclude and request.url.find(self.cache_file_exclude) >= 0 else 0
            if exists_file:
                if exclude:
                    if self.cache_file_exclude_ts >= 0:
                        if time.time() - Fh.get_update_ts(cache_file_path) < self.cache_file_exclude_ts:
                            return True
                        else:
                            logger.debug(
                                f"缓存文件过期。不读取。{cache_file_path}{Fh.get_update_ts(cache_file_path)}  {request.url}")
                            return False
                    # 排除URL无过期时间,就是排除URL都不读缓存
                    else:
                        logger.debug(f"排除的，没设置过期时间，全不读缓存。{request.url}")
                        return False
                else:
                    # 全部时间
                    if self.cache_file_ts:
                        # 全部URL 没有过期
                        if time.time() - Fh.get_update_ts(cache_file_path) < self.cache_file_ts:
                            return True
                        else:
                            logger.debug(f"== OUT TIME，不读缓存。{request.url}")
                            return False
                    else:
                        return True
            else:
                # logger.debug(f"缓存文件不存在。 {request.url}")
                return False

        if should_read_cache(request, spider):
            try:
                logger.info(f"===Read cache===  {cache_file_path.split(os.sep)[-1]}   {request.url}")
                return pickle.load(open(cache_file_path, "rb"))
            except Exception as e:
                logger.info(f"== Can't Read cache ===  Reason is {e}")
                Fh.remove_file(cache_file_path)
                traceback.print_exc()
        else:
            # 不读缓存
            # logger.info("===Shouldn't Read Cache===   " + request.url)
            pass

    # 写文件缓存
    def process_response(self, request, response, spider) -> process_response_type:
        cache_file_path = self.cache_file_path_func(
            url=str(request.url), spider=str(spider.name),
            root_path_cache=self.settings.get("ROOT_PATH_CACHE"),
            request=request)

        # 是否应该重写缓存
        def should_write_cache(request, spider) -> bool:
            exists_file = Fh.is_file_exit(cache_file_path)
            exclude = 1 if self.cache_file_exclude and request.url.find(self.cache_file_exclude) >= 0 else 0
            if exists_file:
                if exclude:
                    if self.cache_file_exclude_ts:
                        if time.time() - Fh.get_update_ts(cache_file_path) > self.cache_file_exclude_ts:
                            logger.debug(
                                f"排除缓存文件过期。需要重写。{cache_file_path}{Fh.get_update_ts(cache_file_path)}  {request.url}")
                            return True
                        else:
                            logger.debug(
                                f"排除的URL没过期，不重写。过了{int(time.time() - Fh.get_update_ts(cache_file_path))} 秒？ {cache_file_path}")
                            return False
                    else:
                        logger.debug(f"排除URL没设置过期时间 都要重写。 {cache_file_path}")
                        return True
                else:
                    # logger.debug(f"不是排除的URL不重写缓存。 {cache_file_path}")
                    return False
            else:
                # logger.debug(f"缓存文件不存在。写缓存。 {request.url}")
                return True

        if response.status < 300:
            Fh.mkdir(Fh.get_file_path_and_name(cache_file_path)[0])
            try:
                if should_write_cache(request, spider):
                    if hasattr(response, "certificate"):
                        del response.certificate
                    Fh.remove_file(cache_file_path)
                    pickle.dump(response, open(cache_file_path, "wb"))
                    logger.info(f"===Write cache===      {cache_file_path}   {request.url}")
                # else:
                #     logger.info(f"===Cache Exists===      {cache_file_path}   {request.url}")
            except Exception as e:
                logger.info(f"== Can't Write cache 2 file==   Reason is {e}")
                Fh.remove_file(cache_file_path)
                # traceback.print_exc()
        else:
            logger.info(f"===Status Code ERROR ===   Code is {response.status} " + response.url)
        return response


# Redis Hash缓存中间件
class CacheRedisHashRequest(object):

    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def __init__(self):
        import XX.DB.RedisHelper as RH
        self.conn_redis7 = RH.RedisHelper.get_redis_connect(self.settings.get("REDIS_HOST"),
                                                            port=self.settings.get("REDIS_PORT"),
                                                            password=self.settings.get("REDIS_PWD"), db=7)

    # 读redis缓存
    def process_request(self, request, spider) -> process_request_type:
        if self.conn_redis7.exists(request.url):
            print("==Read cache 2 redis==")
            url = self.conn_redis7.hget(request.url, "url")
            html = self.conn_redis7.hget(request.url, "html")
            status_code = self.conn_redis7.hget(request.url, "status_code")
            encoding = self.conn_redis7.hget(request.url, "encoding")
            return TextResponse(url=url, body=html, status=status_code, encoding=encoding)

    # 写redis缓存
    def process_response(self, request, response, spider) -> process_response_type:
        if response.status == 200:
            try:
                if not self.conn_redis7.exists(response.url):
                    self.conn_redis7.hset(response.url, "url", response.url)
                    self.conn_redis7.hset(response.url, "html", response.text)
                    self.conn_redis7.hset(response.url, "status_code", response.status)
                    self.conn_redis7.hset(response.url, "encoding", response.encoding)
                    self.conn_redis7.hset(response.url, "spider", spider.name)
                    self.conn_redis7.hset(response.url, "project_name", self.settings.get("PROJECT_NAME", "crawl"))
                    print("==Write cache 2 redis==")
            except:
                print("==Can't Write cache 2 redis==")
                traceback.print_exc()
        return response


# Redis缓存中间件
class CacheRedisRequest(object):
    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def __init__(self):
        import XX.DB.RedisHelper as RH
        self.conn_redis14 = RH.RedisHelper.get_redis_instance()

    # 读redis缓存
    def process_request(self, request, spider) -> process_request_type:
        try:
            content = self.conn_redis14.get(request.url)
            if content:
                try:
                    return pickle.loads(content)
                except Exception as e:
                    self.conn_redis14.delete(request.url)
                    logger.info(e)
        except:
            print("==Can't Read cache from redis==")
            traceback.print_exc()

    # 写redis缓存
    def process_response(self, request, response, spider) -> process_response_type:
        if response.status == 200:
            try:
                if not self.conn_redis14.exists(response.url):
                    pickinfo = pickle.dumps(response)
                    self.conn_redis14.set(response.url, pickinfo)
            except:
                print("==Can't Write cache 2 redis==")
                traceback.print_exc()
        return response


# HBase缓存下载中间件
class CacheHappyBaseRequest(object):
    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def __init__(self):
        pool = happybase.ConnectionPool(size=3, host=self.settings.get("HBASE_HOST", "localhost"),
                                        port=self.settings.get("HBASE_PORT", 9090), protocol='binary',
                                        transport='buffered')
        self.connection = pool._acquire_connection()
        self.table_name = "crawl_" + self.settings.get("PROJECT_NAME", "crawl")
        self.table = self.connection.table(self.table_name)

        # 判断是否有表，没有则生成
        tables = self.connection.tables()
        if self.table_name.encode("utf-8") not in tables:
            self.connection.create_table(self.table_name, {'source': {}, 'data': {}})

    # 读取缓存
    def getCacheResult(self, request, spider, row_key=None):
        if not row_key:
            row_key = spider.name + "_" + Enc.Encrypt.md5(request.url)
        try:
            row = self.table.row(row_key, include_timestamp=True)
            if row:
                # 默认HTML是永久存储的
                if (request.url.endswith("html") or request.url.endswith("htm")) and not self.settings.get(
                        "HBASE_HTML_EXPIRE", 0):
                    return row
                expire = int(self.settings.get("HBASE_EXPIRE_TS", 0))
                if expire > 0:
                    if time.time() - row.get(b'source:url')[1] // 1000 <= expire:
                        return row
                    else:
                        logger.info(str(row.get(b'source:url')[1]) + "\tis out time\t" + str(expire))
                else:
                    return row
        except:
            self.__init__()
            logger.info("Reconnect 2 HBase" + "===" * 33)
        return None

    # 读缓存，生成response
    def process_request(self, request, spider) -> process_request_type:
        row_key = spider.name + "_" + Enc.Encrypt.md5(request.url)
        result = self.getCacheResult(request, spider, row_key=row_key)
        if result:
            logger.info("R>>>>\trow_key\t" + str(row_key) + "\t" + request.url)
            encoding = result.get(b"source:encoding", [b"utf-8"])[0].decode("utf-8")
            return TextResponse(url=result.get(b"source:url")[0].decode("utf-8"),
                                body=result.get(b"source:html")[0].decode("utf-8"),
                                status=result.get(b"source:status_code")[0].decode("utf-8"), encoding=encoding)
        else:
            pass

    # 写缓存到HBASE
    def process_response(self, request, response, spider) -> process_response_type:
        if not self.getCacheResult(request, spider):
            if response.status == 200:
                data = {
                    "source:url": str(response.url),
                    "source:status_code": str(response.status),
                    "source:html": str(response.text),
                    "source:type": "html",
                    "source:size": str(len(response.text)),
                    "source:encoding": response.encoding
                }
                row_key = spider.name + "_" + Enc.Encrypt.md5(request.url)
                try:
                    self.table.put(row_key, data)
                except:
                    self.__init__()
                    logger.info("Reconnect 2 HBase" + "===" * 33)
                logger.info("W<<<<\trow_key\t" + str(row_key) + "\t" + response.url)
            else:
                logger.info("Not 200 ---\t" + str(response.status) + "\t" + response.url)
        return response

    def close_spider(self):
        pass


# HBase缓存下载中间件
class CacheHBaseRK(object):
    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def __init__(self):
        pool = happybase.ConnectionPool(size=3, host=self.settings.get("HBASE_HOST", "localhost"),
                                        port=self.settings.get("HBASE_PORT", 9090), protocol='binary',
                                        transport='buffered')
        self.connection = pool._acquire_connection()
        self.table_name = "crawl_" + self.settings.get("PROJECT_NAME", "crawl")
        self.table = self.connection.table(self.table_name)

        # 判断是否有表，没有则生成
        tables = self.connection.tables()
        if self.table_name.encode("utf-8") not in tables:
            self.connection.create_table(self.table_name, {'source': {}, 'data': {}})

    # 读取缓存
    def getCacheResult(self, request, spider, row_key=None):
        if not row_key:
            row_key = self.settings.get("HBASE_ROW_KEY_FUN")(request=request)
            print("RRRRRRRRRRKKKKKKKK", row_key)
        try:
            row = self.table.row(row_key, include_timestamp=True)
            if row:
                # 默认HTML是永久存储的
                if (request.url.endswith("html") or request.url.endswith("htm")) and not self.settings.get(
                        "HBASE_HTML_EXPIRE", 0):
                    return row
                expire = int(self.settings.get("HBASE_EXPIRE_TS", 0))
                if expire > 0:
                    if time.time() - row.get(b'source:url')[1] // 1000 <= expire:
                        return row
                    else:
                        logger.info(str(row.get(b'source:url')[1]) + "\tis out time")
                else:
                    return row
        except:
            self.__init__()
            logger.info("Reconnect 2 HBase" + "===" * 33)
        return None

    # 读缓存，生成response
    def process_request(self, request, spider) -> process_request_type:
        row_key = self.settings.get("HBASE_ROW_KEY_FUN")(request=request)
        result = self.getCacheResult(request, spider, row_key=row_key)
        if result:
            logger.info("R>>>>\trow_key\t" + str(row_key) + "\t" + request.url)
            encoding = result.get(b"source:encoding", [b"utf-8"])[0].decode("utf-8")
            return TextResponse(url=result.get(b"source:url")[0].decode("utf-8"),
                                body=result.get(b"source:html")[0].decode("utf-8"),
                                status=result.get(b"source:status_code")[0].decode("utf-8"), encoding=encoding)

    # 写缓存到HBASE
    def process_response(self, request, response, spider) -> process_response_type:
        if not self.getCacheResult(request, spider):
            if response.status == 200:
                data = {
                    "source:url": str(response.url),
                    "source:status_code": str(response.status),
                    "source:html": str(response.text),
                    "source:type": "html",
                    "source:size": str(len(response.text)),
                    "source:encoding": response.encoding
                }
                row_key = self.settings.get("HBASE_ROW_KEY_FUN")(request=request)
                try:
                    self.table.put(row_key, data)
                except:
                    self.__init__()
                    logger.info("Reconnect 2 HBase" + "===" * 33)
                logger.info("W<<<<\trow_key\t" + str(row_key) + "\t" + response.url)
            else:
                logger.info("Not 200 --- " + "\t" + response.url)
        return response



# -  *  - coding:utf-8 -  *  -
import time


def pline(line="===", num=50):
    print(line * num)


def p(msg="", ts=0, line=""):
    try:
        if line:
            print(str(msg) + line * 30)
        else:
            print(str(msg))
    except:
        print(" \n === Can't print msg" + str(type(msg)) + " ========== ===")
    time.sleep(ts)


def d(msg="", ts=0, line1="", line2="", msg_action=None, *arg, **kw):
    if line1:
        if not line2:
            line2 = line1
        pline(line1, 30)
    try:
        if msg_action:
            print(msg_action(msg, *arg, **kw))
        else:
            print(msg)
    except:
        print(" \n === Can't print msg" + str(type(msg)) + " ========== ===")
        return
    if line2:
        pline(line2, 30)
    time.sleep(ts)


def pe(*arg, **kwg):
    p(*arg, **kwg)
    exit()


def de(*arg, **kwg):
    d(*arg, **kwg)
    exit()


def dict_print(obj, indent=' '):
    def _pretty(obj, indent):
        for i, tup in enumerate(obj.items()):
            k, v = tup
            # 如果是字符串则拼上""
            if isinstance(k, str):
                k = '"%s"' % k
            if isinstance(v, str):
                v = '"%s"' % v
            # 如果是字典则递归
            if isinstance(v, dict):
                # 计算下一层的indent
                v = ''.join(_pretty(v, indent + ' ' * len(str(k) + ': {')))
            # case,根据(k,v)对在哪个位置确定拼接什么
            if i == 0:  # 开头,拼左花括号
                if len(obj) == 1:
                    yield '{%s: %s}' % (k, v)
                else:
                    yield '{%s: %s,\n' % (k, v)
            elif i == len(obj) - 1:  # 结尾,拼右花括号
                yield '%s%s: %s}' % (indent, k, v)
            else:  # 中间
                yield '%s%s: %s,\n' % (indent, k, v)

    return ''.join(_pretty(obj, indent))


if __name__ == "__main__":
    d("测试 信息    ", line1="-v-", line2="-^-")



# -*- coding: utf-8 -*-
import json
import pickle
import traceback

import requests
from logzero import logger

import XX.Encrypt.EncryptHelper as Enc
import XX.String.StringHelper as String
from XX.File.FileHelper import *
from XX.File.FileHelper import FileHelper as Fh
from XX.Model.Object.ResponseObj import *

requests.packages.urllib3.disable_warnings()


class RequestHelper(object):

    @staticmethod
    def send_request(url, *args, **kw) -> ResponseObj:
        """
        基础发送请求
        Args:
            url: url地址
            *args:
            **kw:

        Returns:
            response
        """
        response = ResponseObj()
        response.status_code = 0
        response.status = 0
        method = str(kw["method"]).lower() if "method" in kw.keys() else "get"
        kw = dict(kw)
        response.url = url
        data = kw["data"] if "data" in kw.keys() else None
        headers = kw["headers"] if "headers" in kw.keys() else None
        cookies = kw["cookies"] if "cookies" in kw.keys() else None
        proxies = kw["proxies"] if "proxies" in kw.keys() else None
        timeout = kw["timeout"] if "timeout" in kw.keys() else None
        verify = kw["verify"] if "verify" in kw.keys() else True
        json_ = kw["json"] if "json" in kw.keys() else None
        allow_redirects = kw["allow_redirects"] if "allow_redirects" in kw.keys() else True
        try:
            if method.lower() in ["get", "post", "head", "delete", "put", "options", 'patch']:
                func = getattr(requests, method.lower())
            else:
                logger.debug(f" == Unsupported  method : {method}")
                response.msg = f"unsupported method : {method}."
                response.method = method
                return response
            # logger.debug("start request")
            request = func(url, data=data, headers=headers, cookies=cookies, proxies=proxies,
                           timeout=timeout, verify=verify, allow_redirects=allow_redirects, json=json_)
            # logger.debug("end request")
            request.encoding = request.apparent_encoding
            response.status = response.status_code = request.status_code
            response.text = request.text
            response.content = request.content
            if kw.get("serialization_type") == "json":
                try:
                    response.text = request.json()
                except Exception as e:
                    response.msg = f"Response is not json! Exception : {e}"
                    pass
            if request.cookies:
                response.cookies = requests.utils.dict_from_cookiejar(request.cookies)
            response.RequestHeaders = headers
            response.encoding = request.encoding
            response.data = data
            response.json = json_
            if request.headers:
                response.ResponseHeaders = dict(request.headers)
            response.headers = response.ResponseHeaders
            if 500 <= request.status_code < 600:
                response.msg = "500-600"
            elif 400 >= request.status_code >= 200:
                response.msg = "successful" + response.msg if response.msg else ""
            else:
                response.msg = "Not normal status"
        except Exception as e:
            # traceback.print_exc()
            logger.debug(e)
            response.status = response.status_code = 0
            response.msg = f"Time out!  Exception : {e}"
        # logger.debug("end request")
        return response

    # 删除缓存，重新请求
    @staticmethod
    def send_refresh_cache_request(url, *arg, **kw) -> ResponseObj:
        """
        强制请求并删除缓存。
        Args:
            url:
            *arg:
            **kw:

        Returns:

        """
        # 写缓存
        if kw.get("save_path"):
            FileHelper.remove_file(kw.get("save_path"))
            logger.debug(f"Delete old cache file : {kw.get('save_path')} ")
        resp = RequestHelper.send_request(url, *arg, **kw)
        if resp.status_code < 300:
            try:
                # logger.debug(f"Pickle cache file ")
                FileHelper.mkdir(FileHelper.get_file_path_and_name(kw.get("save_path"))[0])
                # 保存HTML
                if kw.get("save_html"):
                    open(kw.get("save_path") + ".xhtml", "w", encoding="utf-8").write(resp.text)
                    if kw.get("serialization_type") == "json":
                        _ = open(kw.get("save_path"), "w", encoding="utf-8")
                        json.dump(String.bytes2str(resp.__dict__), _, ensure_ascii=False)
                else:
                    resp.cache = 1
                    resp.cache_file = kw.get("save_path")
                    if kw.get("save_path") and resp.text:
                        pickle.dump(resp, open(kw.get("save_path"), "wb"))
                    else:
                        logger.debug(f"Don't save. No HTML or no cache_path .Code is {resp.status_code}")
                        pass
                return resp
            except Exception as e:
                traceback.print_exc()
                logger.debug("===Can't dump html file to >> " + str(kw.get("save_path")) + f"    {e}")
        else:
            logger.debug(f"Not 200:{resp.status_code}.  {url}")
            return resp

    # 缓存文件请求
    @staticmethod
    def send_cache_request(url, *arg, **kw) -> ResponseObj:
        resp = ResponseObj()
        del_cache_out_ts = kw["del_cache_out_ts"] if "del_cache_out_ts" in kw.keys() else None

        def out_time(fp=kw.get("save_path"), ts=del_cache_out_ts) -> bool:
            # 没设置过期时间
            if not ts:
                return False
            # 文件不存在
            if not FileHelper.is_file_exit(fp):
                return True
            if time.time() - FileHelper.get_create_ts(fp) > ts:
                return True
            return False

        # 存在，且不过期
        if kw.get("save_path") and os.path.isfile(kw.get("save_path")) and os.path.getsize(
                kw.get("save_path")) and not out_time():
            # logger.debug("not out time")
            try:
                if kw.get("serialization_type") == "json":
                    resp.__dict__ = dict(json.load(open(kw.get("save_path"), encoding="utf-8")))
                    resp.__dict__["cache"] = 2
                    resp.__dict__["cache_file"] = kw.get("save_path")
                else:
                    try:
                        resp = pickle.load(open(kw.get("save_path"), "rb"))
                        resp.cache = 2
                        resp.cache_file = kw.get("save_path")
                        if not resp.text or resp.status_code != 200:
                            logger.warning("response 没有内容，重新请求。")
                            return RequestHelper.send_refresh_cache_request(url, *arg, **kw)
                    except Exception as e:
                        traceback.print_exc()
                        resp.msg = f"Failed to unpickle cached content: {e}!"
            except Exception as e:
                traceback.print_exc()
                resp.msg = f"Failed to read cached content: {e}!"
        else:
            # logger.debug("out time!")
            resp = RequestHelper.send_refresh_cache_request(url, *arg, **kw)
        return resp

    @staticmethod
    # 检测缓存文件是否存在
    def send_check_request(url, *arg, **kw):
        if kw.get("save_path") and os.path.isfile(kw.get("save_path")) and os.path.getsize(kw.get("save_path")):
            return 1

    @staticmethod
    def async_request(urls, err_callback=None, pool_size=5, *args, **kw):
        import grequests
        tasks = [grequests.get(u, **kw) for u in urls]
        return grequests.map(tasks, size=pool_size, exception_handler=err_callback)

    @staticmethod
    def try_request_url(url, *arg, **kw) -> ResponseObj:
        """
        可重试的请求。 TODO,使用try注解
        Args:
            url:
            *arg:
            **kw:

        Returns:

        """
        try_times_threshold = kw["try_times_threshold"] if "try_times_threshold" in kw.keys() else 5
        try_times = kw["try_times"] if "try_times" in kw.keys() else 0
        try_ts = kw["try_ts"] if "try_ts" in kw.keys() else 0.01
        kw["try_times"] = try_times
        time.sleep(try_times * try_ts)
        response = RequestHelper.send_cache_request(url, *arg, **kw)
        if 400 <= response.status_code <= 600:
            kw["try_times"] += 1
            logger.debug(" =" + str(response.status_code) + "= ")
            if try_times_threshold and kw["try_times"] >= try_times_threshold:
                return response
            RequestHelper.try_request_url(url, *arg, **kw)
        else:
            return response
        return response

    @staticmethod
    def pick_2_html(pick_path, html_path=""):
        import pickle
        try:
            resp = pickle.load(open(pick_path, "rb"))
            if not html_path:
                html_path = pick_path.split(".")[0] + ".html"
            open(html_path, "w", encoding="utf-8").write(resp.text)
        except Exception as e:
            return


def cache_response(response, roo_path=None):
    if hasattr(response, "cache_file") and response.cache_file:
        pickle.dump(response, open(response.cache_file, "wb"))
    else:
        fp = roo_path + FileHelper.get_md5_name(response.url + Enc.Encrypt.md5(str(response.headers))) + ".pickle"
        FileHelper.mkdir(FileHelper.get_file_path_and_name(fp)[0])
        pickle.dump(response, open(fp, "wb"))
    return 1


def downloader(url, root_path=None, del_cache_out_ts=None, **kwargs):
    save_path = f"{root_path}{Fh.get_md5_name(url)}.pickle" if root_path else None
    return RequestHelper.send_cache_request(url, save_path=save_path, del_cache_out_ts=del_cache_out_ts, **kwargs)


if __name__ == "__main__":
    # url = "https://www.baidu.com/?tn=98010089_dg"
    # save_path = "E:\\baidu.pick"
    # req = RequestHelper.try_request_url(url, save_path=save_path, refresh=1, verify=False, method="get")
    # response = ResponseObj(status_code=0, url="1.com", headers={"kl": "v2"})
    # r = cache_response(response, roo_path="./")
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/9/29 15:11
# @Author   : Peter
# @Des       : 
# @File        : CompanyTaxModel
# @Software: PyCharm
from XX.Model.SqlAlchemy.BaseModel import *
from sqlalchemy import Column, Integer, String, Text, func
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class CompanyTaxModel(Base, BaseModel):
    __tablename__ = 'company_tax'

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer)
    web_key = Column(String(50))
    Name = Column(String(255))
    CreditCode = Column(String(255))
    Address = Column(String(255))
    PhoneNumber = Column(String(255))
    Bank = Column(String(255))
    Bankaccount = Column(String(255))
    is_del = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.Address = kw.get("Address", None)
        self.Bank = kw.get("Bank", None)
        self.Bankaccount = kw.get("Bankaccount", None)
        self.CreditCode = kw.get("CreditCode", None)
        self.Name = kw.get("Name", None)
        self.PhoneNumber = kw.get("PhoneNumber", None)
        self.com_id = kw.get("com_id", None)
        self.web_key = kw.get("web_key", None)
        self.create_ts = kw.get("create_ts", None)
        self.get = kw.get("get", None)
        self.getAll = kw.get("getAll", None)
        self.getAllIds = kw.get("getAllIds", None)
        self.getByFromId = kw.get("getByFromId", None)
        self.getByFromIdAndMod = kw.get("getByFromIdAndMod", None)
        self.getByName = kw.get("getByName", None)
        self.getColumsByFromIdAndMod = kw.get("getColumsByFromIdAndMod", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.metadata = kw.get("metadata", None)
        self.updateId = kw.get("updateId", None)
        self.update_ts = kw.get("update_ts", None)


if __name__ == '__main__':
    createInitFunction(CompanyTaxModel)



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2018/8/17 0:28
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : City.py
# @Software : PyCharm

from sqlalchemy import Column, Integer, String
from sqlalchemy.ext.declarative import declarative_base

from .BaseModel import *

Base = declarative_base()
metadata = Base.metadata


class TestModel(Base, BaseModel):
    __tablename__ = 'test'

    id = Column(Integer, primary_key=True)
    name = Column(String(255))
    age = Column(Integer)
    is_del = Column(Integer)
    cts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.id = kw.get("id", None)
        self.name = kw.get("name", None)
        self.age = kw.get("age", None)
        self.is_del = kw.get("is_del", None)
        self.cts = kw.get("cts", None)


if __name__ == '__main__':
    createInitFunction(TestModel)



# -*- coding:utf-8 -*-
from sqlalchemy import Column, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from XX.Model.SqlAlchemy.BaseModel import BaseModel

Base = declarative_base()
metadata = Base.metadata


class WxCountInfo(Base, BaseModel):
    __tablename__ = 'wx_count_info'

    id = Column(Integer, primary_key=True)
    count_id = Column(Integer)
    description = Column(String(255))
    company = Column(String(255))
    register_date = Column(String(255))
    qr = Column(String(255))
    is_del = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.company = kw.get("company", None)
        self.count_id = kw.get("count_id", None)
        self.create_ts = kw.get("create_ts", None)
        self.description = kw.get("description", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.metadata = kw.get("metadata", None)
        self.qr = kw.get("qr", None)
        self.register_date = kw.get("register_date", None)
        self.update_ts = kw.get("update_ts", None)

    @staticmethod
    def getInfoByCountId(count_id, session):
        infos = session.query(WxCountInfo).filter(WxCountInfo.count_id == count_id).all()
        return infos[0].id if infos else None


if __name__ == '__main__':
    import types

    print("""\n\n\ndef __init__(self, *arg, **kw):""")
    for k in dir(WxCountInfo):
        if not (str(k).startswith("_") or isinstance(getattr(WxCountInfo, k), types.FunctionType)):
            print("""    self.{attr} = kw.get("{attr}", None)""".format(attr=k))



#!/usr/bin/env python
# encoding: utf-8
import os
import unittest

from scrapy.http.request import Request
from scrapy.http.response.text import TextResponse
from scrapy.spiders import Spider

import XX.File.FileHelper as Fh
from XX.Scrapy.DM.CacheDM import CacheFileRequest


def get_cache_file(url, spider=None, root_path_cache_=""):
    return root_path_cache_ + spider + os.sep + Fh.FileHelper.get_md5_name(url) + ".cache"


class TestCacheDM(unittest.TestCase):
    url = ""

    def setUp(self) -> None:
        self.url = "https://www.li2st1.com"
        self.cache_file_exclude = "list"
        self.cache_file_exclude_ts = 86400

        self.cfr = CacheFileRequest()
        self.cfr.cache_file_exclude = self.cache_file_exclude
        self.cfr.cache_file_exclude_ts = self.cache_file_exclude_ts
        self.cfr.cacheFilePath = get_cache_file

        self.request = Request(url=self.url, body="111")
        self.spider = Spider("name")

    # 在response里写缓存
    def test_write(self):
        self.response = TextResponse(url=self.url, body="OK".encode("utf-8"))
        resp = self.cfr.process_response(self.request, self.response, self.spider)
        self.assertIsInstance(resp, TextResponse)

    # 在request里读缓存
    def test_read(self):
        resp = self.cfr.process_request(self.request, self.spider)
        self.assertIn(type(resp), [None, TextResponse])


if __name__ == '__main__':
    unittest.main()



from sqlalchemy import Column, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from ..BaseModel import BaseModel

Base = declarative_base()
metadata = Base.metadata


class KeyWordModel(Base, BaseModel):
    __tablename__ = 'keywords'

    id = Column(Integer, primary_key=True)
    key = Column(String(255))

    def __init__(self, *arg, **kw):
        self.key = kw.get("key", None)

    @staticmethod
    def saveKeyWord(data, session):
        key = data.get("key")
        if key:
            exists = KeyWordModel.getByKV("key", key, session)
        else:
            return
        if not exists:
            keyword = KeyWordModel(**data)
            KeyWordModel.addModel(keyword, session)
            return keyword.key
        return exists[0].key



# -*- coding:utf-8 -*-
import json


# 服务端生成的response json结构封装
class ResponseJson(object):
    data = None
    code = 0
    msg = ""

    def __init__(self, data=None, code=0, msg="", status="ok", *arg, **kw):
        self.data = data
        self.code = code
        self.msg = msg

    @staticmethod
    def get_response_json(data=None, code=0, msg="", resformat="dict", status="ok", *arg, **kw):
        if not msg:
            if code == 0:
                msg = "ok"
            else:
                msg = "err"

        response_json = {"data": data, "code": code, "msg": msg}
        if resformat == "dict":
            return response_json
        elif resformat == "str":
            return json.dumps(response_json, ensure_ascii=False)
        else:
            return response_json


if __name__ == "__main__":
    res = ResponseJson()
    print(res.__dict__)
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/9/6 10:05
# @Author   : Peter
# @Des       : 
# @File        : CarTip
# @Software: PyCharm
from XX.Model.SqlAlchemy.BaseModel import BaseModel
from sqlalchemy import Column, Integer, String
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class CarDictModel(Base, BaseModel):
    __tablename__ = 'car_dict'

    id = Column(Integer, primary_key=True)
    name = Column(String(255))
    score = Column(String(255))
    is_del = Column(Integer)
    update_ts = Column(Integer)
    create_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.create_ts = kw.get("create_ts", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.metadata = kw.get("metadata", None)
        self.name = kw.get("name", None)
        self.score = kw.get("score", None)
        self.update_ts = kw.get("update_ts", None)


if __name__ == '__main__':
    import types

    print("""def __init__(self, *arg, **kw):""")
    for k in dir(CarDictModel):
        if not (str(k).startswith("_") or isinstance(getattr(CarDictModel, k), types.FunctionType)):
            print("""    self.{attr} = kw.get("{attr}", None)""".format(attr=k))



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2018/9/10 22:54
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : Help.py
# @Software : PyCharm


# 根据json生成sql
def getSqlByDict(d, tb_name="tb_name", tb_comment="COMMENT"):
    s = "\n\nCREATE TABLE `{tb_name}` (\n`id` int(11) NOT NULL AUTO_INCREMENT,\n\n".format(tb_name=tb_name)
    for k, v in d.items():
        if type(v) in [int, float]:
            s += """\t`{key}` int(11) DEFAULT NULL,""".format(key=k if str(k) != "id" else "web_id") + "\n"
        elif type(v) == list:
            s += """\t`{key}` longtext DEFAULT NULL,""".format(key=k if str(k) != "id" else "web_id") + "\n"
        elif type(v) == dict:
            if v.get("id"):
                s += """\t`{key}_web_id`  varchar(255) DEFAULT NULL,""".format(key=k) + "\n"
                print(":WebWebWebWebWeb")
            else:
                s += """\t`{key}` longtext DEFAULT NULL,""".format(key=k if str(k) != "id" else "web_id") + "\n"
        else:
            s += """\t`{key}` varchar(255) DEFAULT NULL,""".format(key=k if str(k) != "id" else "web_id") + "\n"
    s += """
        `is_del` int(11) DEFAULT 0,
        `update_ts` bigint(20) DEFAULT NULL,
        `create_ts` timestamp NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
        PRIMARY KEY (`id`)
        ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COMMENT='{tb_comment}';""".format(tb_comment=tb_comment)
    print(s)


def getCreateTableSqlBySqlDump(fp):
    for line in open(fp, "r", encoding="utf-8"):
        if line.find("INSERT") >= 0 or line.find("--") >= 0 or line.find("/*") >= 0:
            continue
        print(line.strip())


def delRedisKeysByPrefix(prefix, conn):
    for key in conn.keys():
        if str(key).startswith(prefix):
            conn.delete(key)
            print(key)

def getColumnsByDDL(ddl):
    pass
    


if __name__ == '__main__':
    d = {'url': 'https://www.baidu.com/s?ie=utf8&oe=utf8&wd=%E7%9B%91%E6%8E%A7%E5%B9%B3%E5%8F%B0&tn=98010089_dg&ch=2', 'crawl_ts': 1557479433, 'create_ts': '2019-05-10 17:10:33', 'real_url': 'http://www.tjlsty.com.cn/Home/Product/index/pid/3.html#cur_pro', 'snapshot_url': 'http://www.baidu.com/baidu.php?url=af0000aF4eAvq9HI8keayRouCCa3UvfSC4zodAur9WJD614Ui34sRpxjQjxy9ho1ZPmq7XRjQUGzcBbr02fzKbd8LXc1erKSs-uUxPUdk2D2EnY2ujHbOo4nEBUcjmoBRxYSRQ_Khmjcu8ZR_Zt-38ZyFrOvvX0CEtLzN67PBwXBhe2eLEkNwYEAhWYbeBKf-ipcQ9JgjAYoSJAa4s.Db_a9-CkYP7nZeeV2m5wKV6k3Cr55OCf3F8gYoDkEvyNvyUPOQIJyAp7BEW342d0.U1YY0ZDqEraOvnpq1260mywkXHvt3QOmkoLnJ0K9uZ7Y5Hc0TA-W5H00Ijvt3QOm1egP_sKGUHYznjf0u1dEugK1nfKdpHdBmy-bIykV0ZKGujY10APGujYYnH60UgfqnH0krNtknjDLg1nknWwxn1msnfKopHYs0ZFY5HcsP0K-pyfqnHfYn-tznH03r7tzrjRkr7tzrjTLPNtzrjTzn7tzrjbvP7tzrj61PdtzrjR3PfKBpHYznjwxnHRd0AdW5HDsnj7xrjczPjfsnjnsg1Dsn-ts0Z7spyfqn0Kkmv-b5H00ThIYmyTqn0K9mWYsg100ugFM5H00TZ0qnWbYn101PH030A4vTjYsQW0snj0snj0s0AdYTjYs0AwbUL0qn0KzpWYs0Aw-IWdsmsKhIjYs0ZKC5H00ULnqn0KBI1Yz0A4Y5H00TLCq0A71gv-bm1dsTzdWUfKYIgnqnW64nHcvn1n4PHfYnjb4nWmYPsKzug7Y5HDdPHTYP1T1nWbsPWn0Tv-b5H9WuyP-rAcvnj0snhDsuhf0mLPV5Rn3PWFDPHcvPRf3wRDdfHn0mynqnfKsUWYs0Z7VIjYs0Z7VT1Ys0ZGY5H00UyPxuMFEUHYsg1Kxn7ts0Aw9UMNBuNqsUA78pyw15HKxn7tzP1fvrjf4g100TA7Ygvu_myTqn0Kbmv-b5H00ugwGujYVnfK9TLKWm1Ys0ZNspy4Wm1Ys0Z7VuWYkP6KhmLNY5H00uMGC5H00uh7Y5H00XMK_Ignqn0K9uAu_myTqnfK_uhnqn0KEIjYs0AqzTZfqnanscznsc10WnansQW0snj0sn0KWThnqnHmLPWT&word=%E7%9B%91%E6%8E%A7%E5%B9%B3%E5%8F%B0', 'title': '天津联盛天远监控系统_全程一体化服务 '}


    getSqlByDict(d)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/12/13 16:06
# @Email     : billsteve@126.com
# @Des       : 创建Spider
# @File        : __CREATE_SPIDER.
# @Software: PyCharm


def createSpiderCode(config):
    code = """
import json
import os
import sys

import XX.Tools.Debug as d
from XX.DB.RedisHelper import *
from scrapy import cmdline
from scrapy_redis.spiders import RedisSpider

sys.path.append("{project_path}")
import Config.db as db


class {class_name}(RedisSpider):
    handle_httpstatus_list = [429, 402, 200]
    name = '{spider}'

    def __init__(self, *args, **kwargs):
        RedisSpider.__init__(self, *args, **kwargs)
        self.parameter = json.loads(kwargs.pop('parameter', '{{}}'))
        self.conn_redis = RedisHelper.getRedisConnect(host=db.REDIS_HOST, pwd=db.REDIS_PWD)
        self.ar = self.parameter['ar'] if 'ar' in self.parameter else ""

        self.ar = kwargs.pop("ar", 0)
        if self.ar:
            for i in range(0, 1):
                url = "{test_url}"
                res = self.conn_redis.lpush(self.name + ":start_urls", url)
                print(" === Add url ===\t" + str(res) + "\t====================\t" + url, flush=True)

    def parse(self, response):
        if 400 <= response.status < 600:
            if response.status == 429 or response.status == 407 or response.status == 402:
                add_res = self.conn_redis.lpush(self.name + ":start_urls", response.url)
                print("==Proxy error=== Status_code is " + str(response.status) + " Readd 2 redis res\t" + str(add_res), flush=True)
            else:
                print("==Not allowed status code==" + str(response.status), flush=True)
        else:
            yield getattr(ParseUtils, self.name)(response.html, response.url)
            # json API
            try:
                json_data = json.loads(response.text)
                if json_data:
                    json_data["url"] = response.url
                    yield json_data
                else:
                    d.d(response.text, line1="---")
            except Exception as e:
                d.d(e, line1="===")
                self.conn_redis.lpush(self.name + ":start_urls", response.url)
                


if __name__ == '__main__':
    spider = [spider for spider in dir() if spider.endswith("Spider") and spider != "RedisSpider" and spider != "Spider"][0]
    cmd = 'scrapy crawl  ' + getattr(globals()[spider], "name") + " -a ar=1"
    cmdline.execute(cmd.split())
"""
    print(str(code).format(**config))


if __name__ == '__main__':
    config = {
        "class_name": "TestSpider",
        "spider": "test",
        "test_url": "https://www.baidu.com",
        "project_path": "D:\\code\\python\\Qcc",
    }
    createSpiderCode(config)



# -*- coding: utf-8 -*-
import json
import os
import time

from logzero import logger
from thrift.transport import TSocket

import XX.BuiltinFunctions as bf
import XX.DB.SqlAlchemyHelper as sa
import XX.Encrypt.EncryptHelper as enc
import XX.File.FileHelper as cf
import XX.HTML.HtmlHelper as chtml


# File pipeline:放到今日文件中
class FilePipeline(object):

    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.cacheFilePath = settings.get("FUN_CACHE_FILE_PATH")
        cls.settings = settings
        return cls()

    def process_item(self, item, spider):
        # 数据处理
        item = chtml.parse_dict(item)
        today = time.strftime("%Y_%m_%d", time.localtime(int(time.time())))
        json_str = json.dumps(item, ensure_ascii=False)

        # 保存数据到文件
        file_path = self.settings.get("ROOT_PATH_JSON") + spider.name + os.sep + today + ".json"
        cf.FileHelper.save_file(file_path, json_str + "\n")
        return item


# 放到MySQL数据库
class MysqlPipeline(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        cls.session = sa.SqlAlchemyHelper.get_session_by_cfg(cls.settings.get("mysql_cfg"))
        return cls()

    def process_item(self, item, spider):
        import importlib
        module = importlib.import_module("Util.Json2Mysql", self.settings.get("PROJECT_PATH"))
        if hasattr(module, spider.name):
            getattr(module, spider.name)(item, self.session)
        else:
            logger.info("No Json2Mysql function")
        return item


# 放到Mongo数据库
class MongoPipeline(object):
    def process_item(self, item, spider):
        return item


# 放到Kakfa队列
class KafkaPipeline(object):
    def __init__(self):
        from pykafka import KafkaClient
        self.client = KafkaClient(hosts="LOCALHOST" + ":6667")

    def process_item(self, item, spider):
        topic_producer = self.client.topics[spider.name]
        producer = topic_producer.get_producer()
        # 数据处理
        item = chtml.parse_dict(item)
        json_str = json.dumps(item, ensure_ascii=False)
        producer.produce(json_str)
        bf.print_from_head(spider.name + "\tAdd kafka")
        return item


class HivePipeline(object):
    def process_item(self, item, spider):
        return item


class SparkPipeline(object):
    def process_item(self, item, spider):
        return item


class StormPipeline(object):
    def process_item(self, item, spider):
        return item


class HBasePipeline(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        return cls()

    def __init__(self):
        self.transport = TSocket.TSocket(self.settings.get("HBASE_HOST", "localhost"), self.settings.get("HBASE_PORT", 9090))
        self.transport.open()
        self.protocol = TBinaryProtocol.TBinaryProtocol(self.transport)
        self.client = Hbase.Client(self.protocol)

        # 判断是否有表，没有则生成
        tables = self.client.getTableNames()
        self.table_name = "crawl_" + self.settings.get("PROJECT_NAME", self.settings.get("BOT_NAME", "crawl"))
        if self.table_name not in tables:
            source = ColumnDescriptor(name='source')
            data = ColumnDescriptor(name='data')
            self.client.createTable(self.table_name, [source, data])

    def process_item(self, item, spider):
        # 保存到crawl_project表中 spider_name+md5(url) rowkey中，data:json_str中
        # crawl_project >  spider_name+md5(url)  > data:json_str
        url = item.get("url")
        if url:
            row = spider.name + "_" + enc.Encrypt.md5(url)
            mutations = list()
            mutations.append(Mutation(column="data:json", value=str(json.dumps(item, ensure_ascii=False))))
            self.client.mutateRow(self.table_name, row, mutations)
            logger.info("Pipeline Data 2 HBase\t" + row)
        else:
            logger.info("No url from spider \t" + spider.name)
        return item

    def close_spider(self, spider):
        self.transport.close()


class TestPipeline(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        return cls()

    def process_item(self, item, spider):
        print("===" * 44)
        print(self.settings)
        print(dir(spider))
        print(dir(self))
        print("===" * 44)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         : 2019/3/21 14:00
# @File           : BaiduSearchResult
# @Des           : 百度搜索结果Item
# @Email        : billsteve@126.com
import time

from scrapy import Item, Field


# 百度搜索结果
class BaiduSearchResultItem(Item):
    # 平台
    platform = Field()
    # 关键字
    keyword = Field()
    # 爬取时间
    crawl_time = Field()
    # 当前URL
    url = Field()
    # 帖子的真正url
    real_url = Field()
    # 来源的url
    source_url = Field()
    # 帖子名字
    title = Field()
    # 爬虫名字
    spider = Field()
    # 百度跳转URL
    skip_url = Field()
    # 快照URL
    snapshot_url = Field()
    # 快照URL
    show_url = Field()
    # is advertise 是否是广告
    is_ad = Field()
    # content
    content = Field()

    def __init__(self, *args, **kwargs):
        Item.__init__(self, *args, **kwargs)
        self['platform'] = kwargs.get("keyword")
        self['keyword'] = kwargs.get("keyword")
        self['crawl_time'] = int(time.time())
        self['url'] = kwargs.get("url")
        self['real_url'] = kwargs.get("real_url")
        self['title'] = kwargs.get("title")
        self['source_url'] = kwargs.get("title")
        self['spider'] = kwargs.get("spider")
        self['skip_url'] = kwargs.get("skip_url")
        self['snapshot_url'] = kwargs.get("snapshot_url")
        self['show_url'] = kwargs.get("show_url")
        self['is_ad'] = kwargs.get("is_ad")
        self['content'] = kwargs.get("content")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/16 10:19
# @Author   : Peter
# @Des       : 
# @File        : BBSItem
# @Software: PyCharm

import XX.Date.DatetimeHelper as ctime
from scrapy import Item, Field


# 帖子模型
class BBSItem(Item):
    # BBS平台 (车质网论坛', 汽车之家论坛,百度贴吧，易车网论坛，易车网)
    platform = Field()
    # 关键字
    keyword = Field()
    # 插入时间
    insert_time = Field()
    # 帖子的url
    bbs_url = Field()
    # 来源的url
    bbs_source_url = Field()
    # 帖子编号 就是论坛的 web_id
    bbs_id = Field()
    # 帖子名字
    bbs_name = Field()
    # 帖子时间
    bbs_time = Field()
    # 帖子楼层  :  实际上是：content
    bbs_floor = Field()
    # 帖子信息(浏览量, 评论量等)
    bbs_info = Field()
    # 爬虫名字
    spider = Field()
    # parameter keywords
    pkeyword = Field()
    # 搜索关键词
    searchword = Field()
    # Job_id
    job_id = Field()
    # car_id
    car_id = Field()
    # cache_info
    cache_info = Field()

    def __init__(self, *args, **kwargs):
        super(BBSItem, self).__init__(self, *args, **kwargs)
        self['platform'] = ''
        self['keyword'] = ''
        self['insert_time'] = ctime.GetNowTime()
        self['bbs_url'] = ''
        self['bbs_name'] = ''
        self['bbs_time'] = ''
        self['bbs_floor'] = ""
        self['bbs_info'] = {}
        self['bbs_source_url'] = ''
        self["bbs_id"] = ""
        self['spider'] = ''
        self['searchword'] = ''
        self["pkeyword"] = ""
        self["job_id"] = ""
        self["car_id"] = ""
        self["cache_info"] = {
            "ts": 86400 * 3,
            "file": True
        }



# -*- coding: utf-8 -*-
import os


def create_model_by_url(url):
    os.system(f"sqlacodegen mysql+pymysql://{url}")


def create_model_by_config(host, port, user, pwd, db):
    create_model_by_url(f"{user}:{pwd}@{host}:{port}/{db}")


if __name__ == '__main__':
    pass
    # # fp = "C:\\Users\\billsteve\\Desktop\\tmp\\"
    # fp = "../Model/SqlAlchemy/T.py"
    #
    # link = "sqlacodegen mysql+pymysql://root:HBroot21@cd-cdb-f41yw26m.sql.tencentcdb.com:63626/company> "
    # link = "sqlacodegen mysql+pymysql://root:HBroot21@cd-cdb-f41yw26m.sql.tencentcdb.com:63626/data_sheet> "
    # link = "sqlacodegen mysql+pymysql://root:HBroot21@rm-8vbzabox9s9kvomq61o.mysql.zhangbei.rds.aliyuncs.com:3306/data_sheet> "
    # # link = "sqlacodegen mysql+pymysql://root:HBroot21@cd-cdb-f41yw26m.sql.tencentcdb.com:63626/weibo> "
    # link = "sqlacodegen mysql+pymysql://root:rebind1234@zh:3306/zhihu > "
    # link = "sqlacodegen mysql+pymysql://root:rebind1234@zh:3306/money > "
    # # link = "sqlacodegen mysql+pymysql://root:DRsXT5ZJ6Oi55LPQ@localhost:3306/music > "
    # # link = "sqlacodegen mysql+pymysql://root:HBroot21@cd-cdb-f41yw26m.sql.tencentcdb.com:63626/money> "
    # os.system(link + str(fp))
    #
    # content = open(fp, encoding="utf-8").read()
    # # content = content.replace("INTEGER", "Integer").replace("BIGINT", "BigInteger")
    # # content = re.sub("Integer(\(\d+\))", "Integer", content)
    # # content = re.sub("BigInteger(\(\d+\))", "BigInteger", content)
    # # open(fp, "w", encoding="utf-8").write(content)
    # print(content)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/9/29 15:09
# @Author   : Peter
# @Des       : 
# @File        : CompanyInfoModel
# @Software: PyCharm
from XX.Model.SqlAlchemy.BaseModel import *
from sqlalchemy import Column, Integer, String, Text
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class CompanyInfoModel(Base, BaseModel):
    __tablename__ = 'company_info'

    id = Column(Integer, primary_key=True)
    com_id = Column(Integer, index=True)
    web_key = Column(String(255, 'utf8mb4_unicode_ci'))
    logo = Column(String(255, 'utf8mb4_unicode_ci'))
    name = Column(String(255, 'utf8mb4_unicode_ci'))
    tel = Column(String(255, 'utf8mb4_unicode_ci'))
    email = Column(String(255, 'utf8mb4_unicode_ci'))
    web_url = Column(String(255, 'utf8mb4_unicode_ci'))
    Address = Column(String(255, 'utf8mb4_unicode_ci'))
    AnnualReports = Column(String(255, 'utf8mb4_unicode_ci'))
    CreditCode = Column(String(255, 'utf8mb4_unicode_ci'))
    taxpayer_no = Column(String(255, 'utf8mb4_unicode_ci'))
    No = Column(String(255, 'utf8mb4_unicode_ci'))
    organization_no = Column(String(255, 'utf8mb4_unicode_ci'))
    Oper_id = Column(Integer)
    OperName = Column(String(255, 'utf8mb4_unicode_ci'))
    RegistCapi = Column(String(255, 'utf8mb4_unicode_ci'))
    Status = Column(String(255, 'utf8mb4_unicode_ci'))
    StartDate = Column(String(255, 'utf8mb4_unicode_ci'))
    EconKind = Column(String(255, 'utf8mb4_unicode_ci'))
    staff_num = Column(String(255, 'utf8mb4_unicode_ci'))
    TermStart = Column(String(255, 'utf8mb4_unicode_ci'))
    TeamEnd = Column(String(255, 'utf8mb4_unicode_ci'))
    BelongOrg = Column(String(255, 'utf8mb4_unicode_ci'))
    CheckDate = Column(String(255, 'utf8mb4_unicode_ci'))
    en_name = Column(String(255, 'utf8mb4_unicode_ci'))
    register_addr = Column(String(255, 'utf8mb4_unicode_ci'))
    industry_belong = Column(String(255, 'utf8mb4_unicode_ci'))
    used_name = Column(String(255, 'utf8mb4_unicode_ci'))
    location = Column(String(255, 'utf8mb4_unicode_ci'))
    Scope = Column(Text(collation='utf8mb4_unicode_ci'))
    intro = Column(Text(collation='utf8mb4_unicode_ci'))
    EndDate = Column(String(255, 'utf8mb4_unicode_ci'))
    Province = Column(String(50, 'utf8mb4_unicode_ci'))
    Industry = Column(String(255, 'utf8mb4_unicode_ci'))
    ImageUrl = Column(String(255, 'utf8mb4_unicode_ci'))
    OrgNo = Column(String(255, 'utf8mb4_unicode_ci'))
    EnglishName = Column(String(255, 'utf8mb4_unicode_ci'))
    Type = Column(String(255, 'utf8mb4_unicode_ci'))
    Tag = Column(String(255, 'utf8mb4_unicode_ci'))
    Financing = Column(String(255, 'utf8mb4_unicode_ci'))
    DbUpdatedDate = Column(Integer)
    ShortStatus = Column(String(20, 'utf8mb4_unicode_ci'))
    IsExpired = Column(String(5, 'utf8mb4_unicode_ci'))
    DUNSNo = Column(String(25, 'utf8mb4_unicode_ci'))
    TaxNo = Column(String(25, 'utf8mb4_unicode_ci'))
    CbuItem = Column(Text(collation='utf8mb4_unicode_ci'))
    AbuItem = Column(Text(collation='utf8mb4_unicode_ci'))
    OpForm = Column(String(5, 'utf8mb4_unicode_ci'))
    RecCap = Column(String(55, 'utf8mb4_unicode_ci'))
    Liquidation = Column(String(5, 'utf8mb4_unicode_ci'))
    SimpleCancellation = Column(String(5, 'utf8mb4_unicode_ci'))
    CompanyStatus = Column(Integer)
    HoldingType = Column(String(25, 'utf8mb4_unicode_ci'))
    id_del = Column(String(255, 'utf8mb4_unicode_ci'))
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.AbuItem = kw.get("AbuItem", None)
        self.Address = kw.get("Address", None)
        self.AnnualReports = kw.get("AnnualReports", None)
        self.BelongOrg = kw.get("BelongOrg", None)
        self.CbuItem = kw.get("CbuItem", None)
        self.CheckDate = kw.get("CheckDate", None)
        self.CompanyStatus = kw.get("CompanyStatus", None)
        self.CreditCode = kw.get("CreditCode", None)
        self.DUNSNo = kw.get("DUNSNo", None)
        self.DbUpdatedDate = kw.get("DbUpdatedDate", None)
        self.EconKind = kw.get("EconKind", None)
        self.EndDate = kw.get("EndDate", None)
        self.EnglishName = kw.get("EnglishName", None)
        self.Financing = kw.get("Financing", None)
        self.HoldingType = kw.get("HoldingType", None)
        self.ImageUrl = kw.get("ImageUrl", None)
        self.Industry = kw.get("Industry", None)
        self.IsExpired = kw.get("IsExpired", None)
        self.Liquidation = kw.get("Liquidation", None)
        self.No = kw.get("No", None)
        self.OpForm = kw.get("OpForm", None)
        self.Oper_id = kw.get("Oper_id", None)
        self.OperName = kw.get("OperName", None)
        self.OrgNo = kw.get("OrgNo", None)
        self.Province = kw.get("Province", None)
        self.RecCap = kw.get("RecCap", None)
        self.RegistCapi = kw.get("RegistCapi", None)
        self.Scope = kw.get("Scope", None)
        self.ShortStatus = kw.get("ShortStatus", None)
        self.SimpleCancellation = kw.get("SimpleCancellation", None)
        self.StartDate = kw.get("StartDate", None)
        self.Status = kw.get("Status", None)
        self.Tag = kw.get("Tag", None)
        self.TaxNo = kw.get("TaxNo", None)
        self.TeamEnd = kw.get("TeamEnd", None)
        self.TermStart = kw.get("TermStart", None)
        self.Type = kw.get("Type", None)
        self.com_id = kw.get("com_id", None)
        self.web_key = kw.get("web_key", None)
        self.create_ts = kw.get("create_ts", None)
        self.email = kw.get("email", None)
        self.en_name = kw.get("en_name", None)
        self.get = kw.get("get", None)
        self.getAll = kw.get("getAll", None)
        self.getAllIds = kw.get("getAllIds", None)
        self.getByFromId = kw.get("getByFromId", None)
        self.getByFromIdAndMod = kw.get("getByFromIdAndMod", None)
        self.getByName = kw.get("getByName", None)
        self.getColumsByFromIdAndMod = kw.get("getColumsByFromIdAndMod", None)
        self.id = kw.get("id", None)
        self.id_del = kw.get("id_del", None)
        self.industry_belong = kw.get("industry_belong", None)
        self.intro = kw.get("intro", None)
        self.location = kw.get("location", None)
        self.logo = kw.get("logo", None)
        self.metadata = kw.get("metadata", None)
        self.name = kw.get("name", None)
        self.organization_no = kw.get("organization_no", None)
        self.register_addr = kw.get("register_addr", None)
        self.staff_num = kw.get("staff_num", None)
        self.taxpayer_no = kw.get("taxpayer_no", None)
        self.tel = kw.get("tel", None)
        self.updateId = kw.get("updateId", None)
        self.update_ts = kw.get("update_ts", None)
        self.used_name = kw.get("used_name", None)
        self.web_url = kw.get("web_url", None)


if __name__ == '__main__':
    createInitFunction(CompanyInfoModel)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/14 16:15
# @Author   : Peter
# @Site       : 
# @File        : PostItem.py
# @Software: PyCharm
import time

from scrapy import Field, Item


class PostItem(Item):
    # 帖子基本信息----------------------------------------------------------------
    # 爬虫名字
    spider = Field()
    # 平台
    bbs_forum = Field()
    # 论坛
    bbs_catalog = Field()
    # 插入时间
    bbs_crawl_ts = Field()
    # 帖子的url
    bbs_url = Field()
    # 来源的url
    bbs_source_url = Field()
    # 帖子编号
    bbs_web_id = Field()
    # 标题
    bbs_title = Field()
    # 帖子内容
    bbs_content = Field()
    # 帖子时间
    bbs_time = Field()
    # 帖子楼层
    bbs_floor = Field()
    # parameter keywords
    bbs_url_kw = Field()
    # car_id
    bbs_product_id = Field()
    # 论坛ID
    bbs_forum_id = Field()
    # info
    bbs_info = Field()
    # 头图信息
    bbs_pics = Field()
    # 帖子类型
    bbs_type = Field()
    # --------------------------------

    # 帖子附加信息----------------------------------------------------------------
    # 帖子信息(浏览量, 评论量等)
    bbs_read_num = Field()
    bbs_like_num = Field()
    bbs_comment_num = Field()
    bbs_forward_num = Field()

    # 回复ID
    bbs_reply_id = Field()

    # 发布者信息
    bbs_postor_web_id = Field()
    bbs_postor_name = Field()
    bbs_postor_head_pic = Field()
    bbs_postor_homepage = Field()
    bbs_postor_isvip = Field()
    bbs_postor_islocked = Field()
    bbs_postor_info = Field()

    def __init__(self, *arg, **kw):
        super(Item, self).__init__(*arg, **kw)
        # 设置默认值
        # 爬虫名字
        self["spider"] = None
        # 平台
        self["bbs_forum"] = None
        # 平台
        self["bbs_catalog"] = None
        # 插入时间
        self["bbs_crawl_ts"] = int(time.time())
        # 帖子的url
        self["bbs_url"] = None
        # 来源的url
        self["bbs_source_url"] = None
        # 帖子编号
        self["bbs_web_id"] = None
        # 标题
        self["bbs_title"] = None
        # 帖子内容
        self["bbs_content"] = None
        # 帖子时间
        self["bbs_time"] = None
        # 帖子楼层(1是主贴2是第一个回帖)
        self["bbs_floor"] = None
        # parameter keywords
        self["bbs_url_kw"] = None
        # car_id
        self["bbs_product_id"] = None
        # 论坛ID
        self["bbs_forum_id"] = None
        # info
        self["bbs_info"] = None
        # 图片信息
        self["bbs_pics"] = None
        # 帖子类型
        self["bbs_type"] = None
        # 帖子信息(浏览量, 评论量等)
        self["bbs_read_num"] = 0
        self["bbs_like_num"] = 0
        self["bbs_comment_num"] = 0
        self["bbs_forward_num"] = 0
        # 回复ID
        self["bbs_reply_id"] = None
        # 发布者信息
        self["bbs_postor_web_id"] = None
        self["bbs_postor_name"] = None
        self["bbs_postor_head_pic"] = None
        self["bbs_postor_homepage"] = None
        self["bbs_postor_isvip"] = None
        self["bbs_postor_islocked"] = None
        self["bbs_postor_info"] = None



# -*- coding: utf-8 -*-
import re


# TODO：debug 其他图片 以img开头的图片的url

def get_img_urls(html):
    return re.findall('http.+\.jpg', html)


def get_url_from_html(txt):
    return re.findall(r"(?<=href=\").+?(?=\")|(?<=href=\').+?(?=\')", txt)


def get_url_from_text(txt):
    return re.findall(r"http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+", txt)


def get_ip(txt):
    return re.findall(
        r"((?:(?:25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d)))\.){3}(?:25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d))))", txt)


def replace_continuous_blank_2_one(txt):
    return re.sub(' +', ' ', txt)


def get_date(txt):
    if txt and isinstance(txt, str):
        return re.findall(r"(\d{4}-\d{1,2}-\d{1,2})", txt)


def repleaseEmoji(sourceStr, replaceStr=''):
    if isinstance(sourceStr, str):
        try:
            co = re.compile('[\U00010000-\U0010ffff]')
        except re.error:
            co = re.compile('[\uD800-\uDBFF][\uDC00-\uDFFF]')
        return co.sub(sourceStr, replaceStr)
    else:
        return sourceStr


def get_version(app):
    return re.findall(r'\d+\.(?:\d+\.)*\d+', app)


if __name__ == '__main__':
    print(get_version("drupal4"))



# -*- coding: utf-8 -*-
from hashlib import md5


class SimpleHash(object):
    def __init__(self, cap, seed):
        self.cap = cap
        self.seed = seed

    def hash(self, value):
        ret = 0
        for i in range(len(value)):
            ret += self.seed * ret + ord(value[i])
        return (self.cap - 1) & ret


class BloomFilter(object):

    def __init__(self, conn=None, block_num=1, key='bf'):
        """
        :param block_num: one block_num for about 90,000,000; if you have more strings for filtering, increase it.
        :param key: the key's name in Redis
        """
        self.redis_conn = conn
        # <<表示二进制向左移动位数，比如2<<2,2的二进制表示000010，向左移2位，就是001000，就是十进制的8
        self.bit_size = 1 << 31  # Redis的String类型最大容量为512M，现使用256M
        self.seeds = [5, 7, 11, 13, 31, 37, 61]
        self.key = "B_" + key + "_"
        self.block_num = block_num
        self.hash_func = []
        for seed in self.seeds:
            self.hash_func.append(SimpleHash(self.bit_size, seed))

    def is_exists(self, str_input):
        if not str_input:
            return False
        m5 = md5()
        m5.update(str_input.encode("utf-8"))
        str_input = m5.hexdigest()
        ret = True
        name = self.key + str(int(str_input[0:2], 16) % self.block_num)
        for f in self.hash_func:
            loc = f.hash(str_input)
            ret = ret & self.redis_conn.getbit(name, loc)
        return ret

    def add(self, str_input):
        if BloomFilter.is_exists(self, str_input):
            return 0
        m5 = md5()
        m5.update(str_input.encode("utf-8"))
        str_input = m5.hexdigest()
        name = self.key + str(int(str_input[0:2], 16) % self.block_num)
        for f in self.hash_func:
            loc = f.hash(str_input)
            self.redis_conn.setbit(name, loc, 1)
        return 1


if __name__ == '__main__':
    """ 第一次运行时会显示 not exists!，之后再运行会显示 exists! """
    from .DB.RedisHelper import *

    conn_redis = RedisHelper.get_redis_connect("localhost", password=None, db=0)
    bf = BloomFilter(conn_redis)
    if bf.is_exists('http://www.baidu.com'):  # 判断字符串是否存在
        print('exists!')
    else:
        print('not exists!')
        bf.add('http://www.baidu.com')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/24 13:46
# @Author   : Peter
# @Des       : 
# @File        : StringHelper
# @Software: PyCharm
import string


def bytes2str(l):
    if type(l) == bytes:
        return l.decode("utf-8")
    elif type(l) == list or type(l) == tuple:
        res = []
        for one in list(l):
            res.append(bytes2str(one))
        return res
    elif type(l) == dict:
        res = {}
        for k, v in l.items():
            res[k] = bytes2str(v)
        return res
    elif type(l) == str:
        return l
    else:
        return l


# 获取字符的数量
def get_letter_count(s):
    num = 0
    for c in s:
        # 英文
        if c in string.ascii_letters:
            num += 1
    return num


# 获取汉字数量
def get_zh_count(s):
    num = 0
    for c in s:
        # 英文
        if c.isalpha() and c not in string.ascii_letters:
            num += 1
    return num


if __name__ == '__main__':
    s = "我  s 是你的bbaba!"
    print(get_zh_count(s))
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
import os
import time

import XX.File.FileHelper as cf
import XX.HTML.HtmlHelper as chtml


# File pipeline:放到今日文件中
class FilePipeline(object):

    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def process_item(self, item, spider):
        # 数据处理
        # item = chtml.parse_dict(dict(item))
        today = time.strftime("%Y-%m-%d", time.localtime(int(time.time())))
        ym = time.strftime("%Y-%m", time.localtime(int(time.time())))
        json_str = json.dumps(dict(item), ensure_ascii=False)

        # 保存数据到文件
        file_path = self.settings.get("ROOT_PATH_JSON") + spider.name + os.sep + ym.replace("-", os.sep) + os.sep + today + ".json"
        cf.FileHelper.save_file(file_path, json_str + "\n")
        return item



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/9/19 17:30
# @Author   : Peter
# @Des       : 
# @File        : InfoForumCarUrl
# @Software: PyCharm
# coding: utf-8
from sqlalchemy import Column, String, Integer
from sqlalchemy.ext.declarative import declarative_base

from XX.Model.SqlAlchemy.BaseModel import BaseModel

Base = declarative_base()
metadata = Base.metadata


class InfoForumCarUrlModel(Base, BaseModel):
    __tablename__ = 'info_forum_car_url'

    id = Column(Integer, primary_key=True)
    forum_id = Column(Integer)
    car_name = Column(String(255))
    url = Column(String(255))
    is_del = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.car_name = kw.get("car_name", None)
        self.create_ts = kw.get("create_ts", None)
        self.forum_id = kw.get("forum_id", None)
        self.getByFromId = kw.get("getByFromId", None)
        self.getByFromIdAndMod = kw.get("getByFromIdAndMod", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.metadata = kw.get("metadata", None)
        self.update_ts = kw.get("update_ts", None)
        self.url = kw.get("url", None)

    @staticmethod
    def getByForumidAndName(forum_id, car_name, session):
        return session.query(InfoForumCarUrlModel).filter(InfoForumCarUrlModel.forum_id == forum_id, InfoForumCarUrlModel.car_name == car_name).all()

    @staticmethod
    def searchByForumidAndName(forum_id, car_name, session):
        infos = InfoForumCarUrlModel.getByForumidAndName(forum_id, car_name, session)
        if not infos:
            return session.query(InfoForumCarUrlModel) \
                .filter(InfoForumCarUrlModel.forum_id == forum_id, InfoForumCarUrlModel.car_name.like("%" + str(car_name) + "%")) \
                .all()


if __name__ == '__main__':
    import types

    print("""def __init__(self, *arg, **kw):""")
    for k in dir(InfoForumCarUrlModel):
        if not (str(k).startswith("_") or isinstance(getattr(InfoForumCarUrlModel, k), types.FunctionType)):
            print("""    self.{attr} = kw.get("{attr}", None)""".format(attr=k))



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2018/10/11 20:14
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : HeaderHelper.py
# @Software : PyCharm
import user_agent


def create_random_ua(**kw):
    return user_agent.generate_user_agent(**kw)


def create_html_header(**kw):
    header = create_ajax_header(**kw)
    header["Accept"] = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8"
    return header


def create_ajax_header(**kw):
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.5",
        "Cache-Control": "max-age=0",
        "Connection": "keep-alive",
        "Upgrade-Insecure-Requests": "1",
        "User-Agent": str(create_random_ua()),
    }
    return headers


def create_ajax_accept(header, **kw):
    header["Accept"] = "application/json, text/javascript, */*; q=0.01"
    return header


def create_img_header(header=None):
    if not header:
        header = create_ajax_header()
    header["Accept"] = "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8"
    return header


def create_host_header(host, **kw):
    headers = create_ajax_header(**kw)
    headers["Host"] = host
    return headers


def create_referer_header(referer, **kw):
    headers = create_ajax_header(**kw)
    headers["Referer"] = referer
    return headers


def create_host_referer_header(host, referer, **kw):
    headers = create_ajax_header(**kw)
    headers["Host"] = host
    headers["Referer"] = referer
    return headers


def get_header_from_line_str(headers):
    return {each.split(':', 1)[0].strip(): each.split(':', 1)[1].strip()
            for each in headers.split('\n') if each.split()}


def get_cookie_from_set_cookie():
    pass


if __name__ == '__main__':
    import time

    while 1:
        print(create_img_header((create_ajax_header())))
        time.sleep(0.8)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/9/17 17:30
# @Author   : Peter
# @Des       : 
# @File        : CarDetailModel
# @Software: PyCharm
# coding: utf-8
from XX.Model.SqlAlchemy.BaseModel import BaseModel
from sqlalchemy import Column, Integer, String, text
from sqlalchemy.dialects.mysql.types import TINYINT
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class CarDetailModel(Base, BaseModel):
    __tablename__ = 'car_detail'

    id = Column(Integer, primary_key=True)
    car_id = Column(Integer, unique=True)
    car_web_id = Column(Integer)
    car_series_id = Column(Integer)
    car_level_id = Column(Integer)
    year = Column(String(10))
    k_1 = Column(String(250))
    k_2 = Column(String(250))
    k_3 = Column(String(25))
    k_4 = Column(String(25))
    k_5 = Column(String(25))
    k_6 = Column(String(25))
    k_7 = Column(String(25))
    k_8 = Column(String(25))
    k_9 = Column(String(25))
    k_10 = Column(String(25))
    k_11 = Column(String(25))
    k_12 = Column(String(25))
    k_13 = Column(String(25))
    k_14 = Column(String(25))
    k_15 = Column(String(25))
    k_16 = Column(String(25))
    k_17 = Column(String(25))
    k_18 = Column(String(25))
    k_19 = Column(String(25))
    k_20 = Column(String(25))
    k_21 = Column(String(25))
    k_22 = Column(String(25))
    k_23 = Column(String(25))
    k_24 = Column(String(25))
    k_25 = Column(String(25))
    k_26 = Column(String(25))
    k_27 = Column(String(25))
    k_28 = Column(String(25))
    k_29 = Column(String(25))
    k_30 = Column(String(25))
    k_31 = Column(String(25))
    k_32 = Column(String(25))
    k_33 = Column(String(25))
    k_34 = Column(String(25))
    k_35 = Column(String(25))
    k_36 = Column(String(25))
    k_37 = Column(String(25))
    k_38 = Column(String(25))
    k_39 = Column(String(25))
    k_40 = Column(String(25))
    k_41 = Column(String(25))
    k_42 = Column(String(25))
    k_43 = Column(String(25))
    k_44 = Column(String(25))
    k_45 = Column(String(25))
    k_46 = Column(String(25))
    k_47 = Column(String(25))
    k_48 = Column(String(25))
    k_49 = Column(String(25))
    k_50 = Column(String(25))
    k_51 = Column(String(25))
    k_52 = Column(String(25))
    k_53 = Column(String(25))
    k_54 = Column(String(25))
    k_55 = Column(String(25))
    k_56 = Column(String(25))
    k_57 = Column(String(25))
    k_58 = Column(String(25))
    k_59 = Column(String(25))
    k_60 = Column(String(25))
    k_61 = Column(String(25))
    k_62 = Column(String(25))
    k_63 = Column(String(25))
    k_64 = Column(String(25))
    k_65 = Column(String(25))
    k_66 = Column(String(25))
    k_67 = Column(String(25))
    k_68 = Column(String(25))
    k_69 = Column(String(25))
    k_70 = Column(String(25))
    k_71 = Column(String(25))
    k_72 = Column(String(25))
    k_73 = Column(String(25))
    k_74 = Column(String(25))
    k_75 = Column(String(25))
    k_76 = Column(String(25))
    k_77 = Column(String(25))
    k_78 = Column(String(25))
    k_79 = Column(String(25))
    k_80 = Column(String(25))
    k_81 = Column(String(25))
    k_82 = Column(String(25))
    k_83 = Column(String(25))
    k_84 = Column(String(25))
    k_85 = Column(String(25))
    k_86 = Column(String(25))
    k_87 = Column(String(25))
    k_88 = Column(String(25))
    k_89 = Column(String(25))
    k_90 = Column(String(25))
    k_91 = Column(String(25))
    k_92 = Column(String(25))
    k_93 = Column(String(25))
    k_94 = Column(String(25))
    k_95 = Column(String(25))
    k_96 = Column(String(25))
    k_97 = Column(String(25))
    k_98 = Column(String(25))
    k_99 = Column(String(25))
    k_100 = Column(String(25))
    k_101 = Column(String(25))
    k_102 = Column(String(25))
    k_103 = Column(String(25))
    k_104 = Column(String(25))
    k_105 = Column(String(25))
    k_106 = Column(String(25))
    k_107 = Column(String(25))
    k_108 = Column(String(25))
    k_109 = Column(String(25))
    k_110 = Column(String(25))
    k_111 = Column(String(25))
    k_112 = Column(String(25))
    k_113 = Column(String(25))
    k_114 = Column(String(25))
    k_115 = Column(String(25))
    k_116 = Column(String(25))
    k_117 = Column(String(25))
    k_118 = Column(String(25))
    k_119 = Column(String(25))
    k_120 = Column(String(25))
    k_121 = Column(String(25))
    k_122 = Column(String(25))
    k_123 = Column(String(25))
    k_124 = Column(String(25))
    k_125 = Column(String(25))
    k_126 = Column(String(25))
    k_127 = Column(String(25))
    k_128 = Column(String(25))
    k_129 = Column(String(25))
    k_130 = Column(String(25))
    k_131 = Column(String(25))
    k_132 = Column(String(25))
    k_133 = Column(String(25))
    k_134 = Column(String(25))
    k_135 = Column(String(25))
    k_136 = Column(String(25))
    k_137 = Column(String(25))
    k_138 = Column(String(25))
    k_139 = Column(String(25))
    k_140 = Column(String(25))
    k_141 = Column(String(25))
    k_142 = Column(String(25))
    k_143 = Column(String(25))
    k_144 = Column(String(25))
    k_145 = Column(String(25))
    k_146 = Column(String(25))
    k_147 = Column(String(25))
    k_148 = Column(String(25))
    k_149 = Column(String(25))
    k_150 = Column(String(25))
    k_151 = Column(String(25))
    k_152 = Column(String(25))
    k_153 = Column(String(25))
    k_154 = Column(String(25))
    k_155 = Column(String(25))
    k_156 = Column(String(25))
    k_157 = Column(String(25))
    k_158 = Column(String(25))
    k_159 = Column(String(25))
    k_160 = Column(String(25))
    k_161 = Column(String(25))
    k_162 = Column(String(25))
    k_163 = Column(String(25))
    k_164 = Column(String(25))
    k_165 = Column(String(25))
    k_166 = Column(String(25))
    k_167 = Column(String(25))
    k_168 = Column(String(25))
    k_169 = Column(String(25))
    k_170 = Column(String(25))
    k_171 = Column(String(25))
    k_172 = Column(String(25))
    k_173 = Column(String(25))
    k_174 = Column(String(25))
    k_175 = Column(String(25))
    k_176 = Column(String(25))
    k_177 = Column(String(25))
    k_178 = Column(String(25))
    k_179 = Column(String(25))
    k_180 = Column(String(25))
    k_181 = Column(String(25))
    k_182 = Column(String(25))
    k_183 = Column(String(25))
    k_184 = Column(String(25))
    k_185 = Column(String(25))
    k_186 = Column(String(25))
    k_187 = Column(String(25))
    k_188 = Column(String(25))
    k_189 = Column(String(25))
    k_190 = Column(String(25))
    k_191 = Column(String(25))
    k_192 = Column(String(25))
    k_193 = Column(String(25))
    k_194 = Column(String(25))
    k_195 = Column(String(25))
    k_196 = Column(String(25))
    k_197 = Column(String(25))
    k_198 = Column(String(25))
    k_199 = Column(String(25))
    k_200 = Column(String(25))
    k_201 = Column(String(25))
    k_202 = Column(String(25))
    k_203 = Column(String(25))
    k_204 = Column(String(25))
    k_205 = Column(String(25))
    k_206 = Column(String(25))
    k_207 = Column(String(25))
    k_208 = Column(String(25))
    k_209 = Column(String(25))
    k_210 = Column(String(25))
    k_211 = Column(String(25))
    k_212 = Column(String(25))
    k_213 = Column(String(25))
    k_214 = Column(String(25))
    k_215 = Column(String(25))
    k_216 = Column(String(25))
    k_217 = Column(String(25))
    k_218 = Column(String(25))
    k_219 = Column(String(25))
    k_220 = Column(String(25))
    k_221 = Column(String(25))
    k_222 = Column(String(25))
    k_223 = Column(String(25))
    k_224 = Column(String(25))
    k_225 = Column(String(25))
    k_226 = Column(String(25))
    k_227 = Column(String(25))
    k_228 = Column(String(25))
    k_229 = Column(String(25))
    k_230 = Column(String(25))
    k_231 = Column(String(25))
    k_232 = Column(String(25))
    k_233 = Column(String(25))
    k_234 = Column(String(25))
    k_235 = Column(String(25))
    k_236 = Column(String(25))
    k_237 = Column(String(25))
    k_238 = Column(String(25))
    k_239 = Column(String(25))
    k_240 = Column(String(25))
    k_241 = Column(String(25))
    k_242 = Column(String(25))
    k_243 = Column(String(25))
    k_244 = Column(String(25))
    k_245 = Column(String(25))
    k_246 = Column(String(25))
    k_247 = Column(String(25))
    k_248 = Column(String(25))
    k_249 = Column(String(25))
    k_250 = Column(String(25))
    k_251 = Column(String(25))
    k_252 = Column(String(25))
    k_253 = Column(String(25))
    k_254 = Column(String(25))
    k_255 = Column(String(25))
    k_256 = Column(String(25))
    k_257 = Column(String(25))
    k_258 = Column(String(25))
    k_259 = Column(String(25))
    k_260 = Column(String(25))
    k_261 = Column(String(25))
    k_262 = Column(String(25))
    k_263 = Column(String(25))
    k_264 = Column(String(25))
    k_265 = Column(String(25))
    k_266 = Column(String(25))
    k_267 = Column(String(25))
    k_268 = Column(String(25))
    k_269 = Column(String(25))
    k_270 = Column(String(25))
    k_271 = Column(String(25))
    k_272 = Column(String(25))
    k_273 = Column(String(25))
    k_274 = Column(String(25))
    k_275 = Column(String(25))
    k_276 = Column(String(25))
    k_277 = Column(String(25))
    k_278 = Column(String(25))
    k_279 = Column(String(25))
    k_280 = Column(String(25))
    k_281 = Column(String(25))
    k_282 = Column(String(25))
    k_283 = Column(String(25))
    k_284 = Column(String(25))
    k_285 = Column(String(25))
    k_286 = Column(String(25))
    k_287 = Column(String(25))
    k_288 = Column(String(25))
    k_289 = Column(String(25))
    k_290 = Column(String(25))
    k_291 = Column(String(25))
    k_292 = Column(String(25))
    k_293 = Column(String(25))
    k_294 = Column(String(25))
    k_295 = Column(String(25))
    k_296 = Column(String(25))
    k_297 = Column(String(25))
    k_298 = Column(String(25))
    k_299 = Column(String(25))
    k_300 = Column(String(25))
    k_301 = Column(String(25))
    k_302 = Column(String(25))
    k_303 = Column(String(25))
    k_304 = Column(String(25))
    k_305 = Column(String(25))
    k_306 = Column(String(25))
    k_307 = Column(String(25))
    k_308 = Column(String(25))
    k_309 = Column(String(25))
    k_310 = Column(String(25))
    k_311 = Column(String(25))
    k_312 = Column(String(25))
    k_313 = Column(String(25))
    k_314 = Column(String(25))
    is_del = Column(TINYINT(4), server_default=text("'0'"))
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.car_id = kw.get("car_id", None)
        self.car_level_id = kw.get("car_level_id", None)
        self.car_series_id = kw.get("car_series_id", None)
        self.car_web_id = kw.get("car_web_id", None)
        self.create_ts = kw.get("create_ts", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.k_1 = kw.get("k_1", None)
        self.k_10 = kw.get("k_10", None)
        self.k_100 = kw.get("k_100", None)
        self.k_101 = kw.get("k_101", None)
        self.k_102 = kw.get("k_102", None)
        self.k_103 = kw.get("k_103", None)
        self.k_104 = kw.get("k_104", None)
        self.k_105 = kw.get("k_105", None)
        self.k_106 = kw.get("k_106", None)
        self.k_107 = kw.get("k_107", None)
        self.k_108 = kw.get("k_108", None)
        self.k_109 = kw.get("k_109", None)
        self.k_11 = kw.get("k_11", None)
        self.k_110 = kw.get("k_110", None)
        self.k_111 = kw.get("k_111", None)
        self.k_112 = kw.get("k_112", None)
        self.k_113 = kw.get("k_113", None)
        self.k_114 = kw.get("k_114", None)
        self.k_115 = kw.get("k_115", None)
        self.k_116 = kw.get("k_116", None)
        self.k_117 = kw.get("k_117", None)
        self.k_118 = kw.get("k_118", None)
        self.k_119 = kw.get("k_119", None)
        self.k_12 = kw.get("k_12", None)
        self.k_120 = kw.get("k_120", None)
        self.k_121 = kw.get("k_121", None)
        self.k_122 = kw.get("k_122", None)
        self.k_123 = kw.get("k_123", None)
        self.k_124 = kw.get("k_124", None)
        self.k_125 = kw.get("k_125", None)
        self.k_126 = kw.get("k_126", None)
        self.k_127 = kw.get("k_127", None)
        self.k_128 = kw.get("k_128", None)
        self.k_129 = kw.get("k_129", None)
        self.k_13 = kw.get("k_13", None)
        self.k_130 = kw.get("k_130", None)
        self.k_131 = kw.get("k_131", None)
        self.k_132 = kw.get("k_132", None)
        self.k_133 = kw.get("k_133", None)
        self.k_134 = kw.get("k_134", None)
        self.k_135 = kw.get("k_135", None)
        self.k_136 = kw.get("k_136", None)
        self.k_137 = kw.get("k_137", None)
        self.k_138 = kw.get("k_138", None)
        self.k_139 = kw.get("k_139", None)
        self.k_14 = kw.get("k_14", None)
        self.k_140 = kw.get("k_140", None)
        self.k_141 = kw.get("k_141", None)
        self.k_142 = kw.get("k_142", None)
        self.k_143 = kw.get("k_143", None)
        self.k_144 = kw.get("k_144", None)
        self.k_145 = kw.get("k_145", None)
        self.k_146 = kw.get("k_146", None)
        self.k_147 = kw.get("k_147", None)
        self.k_148 = kw.get("k_148", None)
        self.k_149 = kw.get("k_149", None)
        self.k_15 = kw.get("k_15", None)
        self.k_150 = kw.get("k_150", None)
        self.k_151 = kw.get("k_151", None)
        self.k_152 = kw.get("k_152", None)
        self.k_153 = kw.get("k_153", None)
        self.k_154 = kw.get("k_154", None)
        self.k_155 = kw.get("k_155", None)
        self.k_156 = kw.get("k_156", None)
        self.k_157 = kw.get("k_157", None)
        self.k_158 = kw.get("k_158", None)
        self.k_159 = kw.get("k_159", None)
        self.k_16 = kw.get("k_16", None)
        self.k_160 = kw.get("k_160", None)
        self.k_161 = kw.get("k_161", None)
        self.k_162 = kw.get("k_162", None)
        self.k_163 = kw.get("k_163", None)
        self.k_164 = kw.get("k_164", None)
        self.k_165 = kw.get("k_165", None)
        self.k_166 = kw.get("k_166", None)
        self.k_167 = kw.get("k_167", None)
        self.k_168 = kw.get("k_168", None)
        self.k_169 = kw.get("k_169", None)
        self.k_17 = kw.get("k_17", None)
        self.k_170 = kw.get("k_170", None)
        self.k_171 = kw.get("k_171", None)
        self.k_172 = kw.get("k_172", None)
        self.k_173 = kw.get("k_173", None)
        self.k_174 = kw.get("k_174", None)
        self.k_175 = kw.get("k_175", None)
        self.k_176 = kw.get("k_176", None)
        self.k_177 = kw.get("k_177", None)
        self.k_178 = kw.get("k_178", None)
        self.k_179 = kw.get("k_179", None)
        self.k_18 = kw.get("k_18", None)
        self.k_180 = kw.get("k_180", None)
        self.k_181 = kw.get("k_181", None)
        self.k_182 = kw.get("k_182", None)
        self.k_183 = kw.get("k_183", None)
        self.k_184 = kw.get("k_184", None)
        self.k_185 = kw.get("k_185", None)
        self.k_186 = kw.get("k_186", None)
        self.k_187 = kw.get("k_187", None)
        self.k_188 = kw.get("k_188", None)
        self.k_189 = kw.get("k_189", None)
        self.k_19 = kw.get("k_19", None)
        self.k_190 = kw.get("k_190", None)
        self.k_191 = kw.get("k_191", None)
        self.k_192 = kw.get("k_192", None)
        self.k_193 = kw.get("k_193", None)
        self.k_194 = kw.get("k_194", None)
        self.k_195 = kw.get("k_195", None)
        self.k_196 = kw.get("k_196", None)
        self.k_197 = kw.get("k_197", None)
        self.k_198 = kw.get("k_198", None)
        self.k_199 = kw.get("k_199", None)
        self.k_2 = kw.get("k_2", None)
        self.k_20 = kw.get("k_20", None)
        self.k_200 = kw.get("k_200", None)
        self.k_201 = kw.get("k_201", None)
        self.k_202 = kw.get("k_202", None)
        self.k_203 = kw.get("k_203", None)
        self.k_204 = kw.get("k_204", None)
        self.k_205 = kw.get("k_205", None)
        self.k_206 = kw.get("k_206", None)
        self.k_207 = kw.get("k_207", None)
        self.k_208 = kw.get("k_208", None)
        self.k_209 = kw.get("k_209", None)
        self.k_21 = kw.get("k_21", None)
        self.k_210 = kw.get("k_210", None)
        self.k_211 = kw.get("k_211", None)
        self.k_212 = kw.get("k_212", None)
        self.k_213 = kw.get("k_213", None)
        self.k_214 = kw.get("k_214", None)
        self.k_215 = kw.get("k_215", None)
        self.k_216 = kw.get("k_216", None)
        self.k_217 = kw.get("k_217", None)
        self.k_218 = kw.get("k_218", None)
        self.k_219 = kw.get("k_219", None)
        self.k_22 = kw.get("k_22", None)
        self.k_220 = kw.get("k_220", None)
        self.k_221 = kw.get("k_221", None)
        self.k_222 = kw.get("k_222", None)
        self.k_223 = kw.get("k_223", None)
        self.k_224 = kw.get("k_224", None)
        self.k_225 = kw.get("k_225", None)
        self.k_226 = kw.get("k_226", None)
        self.k_227 = kw.get("k_227", None)
        self.k_228 = kw.get("k_228", None)
        self.k_229 = kw.get("k_229", None)
        self.k_23 = kw.get("k_23", None)
        self.k_230 = kw.get("k_230", None)
        self.k_231 = kw.get("k_231", None)
        self.k_232 = kw.get("k_232", None)
        self.k_233 = kw.get("k_233", None)
        self.k_234 = kw.get("k_234", None)
        self.k_235 = kw.get("k_235", None)
        self.k_236 = kw.get("k_236", None)
        self.k_237 = kw.get("k_237", None)
        self.k_238 = kw.get("k_238", None)
        self.k_239 = kw.get("k_239", None)
        self.k_24 = kw.get("k_24", None)
        self.k_240 = kw.get("k_240", None)
        self.k_241 = kw.get("k_241", None)
        self.k_242 = kw.get("k_242", None)
        self.k_243 = kw.get("k_243", None)
        self.k_244 = kw.get("k_244", None)
        self.k_245 = kw.get("k_245", None)
        self.k_246 = kw.get("k_246", None)
        self.k_247 = kw.get("k_247", None)
        self.k_248 = kw.get("k_248", None)
        self.k_249 = kw.get("k_249", None)
        self.k_25 = kw.get("k_25", None)
        self.k_250 = kw.get("k_250", None)
        self.k_251 = kw.get("k_251", None)
        self.k_252 = kw.get("k_252", None)
        self.k_253 = kw.get("k_253", None)
        self.k_254 = kw.get("k_254", None)
        self.k_255 = kw.get("k_255", None)
        self.k_256 = kw.get("k_256", None)
        self.k_257 = kw.get("k_257", None)
        self.k_258 = kw.get("k_258", None)
        self.k_259 = kw.get("k_259", None)
        self.k_26 = kw.get("k_26", None)
        self.k_260 = kw.get("k_260", None)
        self.k_261 = kw.get("k_261", None)
        self.k_262 = kw.get("k_262", None)
        self.k_263 = kw.get("k_263", None)
        self.k_264 = kw.get("k_264", None)
        self.k_265 = kw.get("k_265", None)
        self.k_266 = kw.get("k_266", None)
        self.k_267 = kw.get("k_267", None)
        self.k_268 = kw.get("k_268", None)
        self.k_269 = kw.get("k_269", None)
        self.k_27 = kw.get("k_27", None)
        self.k_270 = kw.get("k_270", None)
        self.k_271 = kw.get("k_271", None)
        self.k_272 = kw.get("k_272", None)
        self.k_273 = kw.get("k_273", None)
        self.k_274 = kw.get("k_274", None)
        self.k_275 = kw.get("k_275", None)
        self.k_276 = kw.get("k_276", None)
        self.k_277 = kw.get("k_277", None)
        self.k_278 = kw.get("k_278", None)
        self.k_279 = kw.get("k_279", None)
        self.k_28 = kw.get("k_28", None)
        self.k_280 = kw.get("k_280", None)
        self.k_281 = kw.get("k_281", None)
        self.k_282 = kw.get("k_282", None)
        self.k_283 = kw.get("k_283", None)
        self.k_284 = kw.get("k_284", None)
        self.k_285 = kw.get("k_285", None)
        self.k_286 = kw.get("k_286", None)
        self.k_287 = kw.get("k_287", None)
        self.k_288 = kw.get("k_288", None)
        self.k_289 = kw.get("k_289", None)
        self.k_29 = kw.get("k_29", None)
        self.k_290 = kw.get("k_290", None)
        self.k_291 = kw.get("k_291", None)
        self.k_292 = kw.get("k_292", None)
        self.k_293 = kw.get("k_293", None)
        self.k_294 = kw.get("k_294", None)
        self.k_295 = kw.get("k_295", None)
        self.k_296 = kw.get("k_296", None)
        self.k_297 = kw.get("k_297", None)
        self.k_298 = kw.get("k_298", None)
        self.k_299 = kw.get("k_299", None)
        self.k_3 = kw.get("k_3", None)
        self.k_30 = kw.get("k_30", None)
        self.k_300 = kw.get("k_300", None)
        self.k_301 = kw.get("k_301", None)
        self.k_302 = kw.get("k_302", None)
        self.k_303 = kw.get("k_303", None)
        self.k_304 = kw.get("k_304", None)
        self.k_305 = kw.get("k_305", None)
        self.k_306 = kw.get("k_306", None)
        self.k_307 = kw.get("k_307", None)
        self.k_308 = kw.get("k_308", None)
        self.k_309 = kw.get("k_309", None)
        self.k_31 = kw.get("k_31", None)
        self.k_310 = kw.get("k_310", None)
        self.k_311 = kw.get("k_311", None)
        self.k_312 = kw.get("k_312", None)
        self.k_313 = kw.get("k_313", None)
        self.k_314 = kw.get("k_314", None)
        self.k_32 = kw.get("k_32", None)
        self.k_33 = kw.get("k_33", None)
        self.k_34 = kw.get("k_34", None)
        self.k_35 = kw.get("k_35", None)
        self.k_36 = kw.get("k_36", None)
        self.k_37 = kw.get("k_37", None)
        self.k_38 = kw.get("k_38", None)
        self.k_39 = kw.get("k_39", None)
        self.k_4 = kw.get("k_4", None)
        self.k_40 = kw.get("k_40", None)
        self.k_41 = kw.get("k_41", None)
        self.k_42 = kw.get("k_42", None)
        self.k_43 = kw.get("k_43", None)
        self.k_44 = kw.get("k_44", None)
        self.k_45 = kw.get("k_45", None)
        self.k_46 = kw.get("k_46", None)
        self.k_47 = kw.get("k_47", None)
        self.k_48 = kw.get("k_48", None)
        self.k_49 = kw.get("k_49", None)
        self.k_5 = kw.get("k_5", None)
        self.k_50 = kw.get("k_50", None)
        self.k_51 = kw.get("k_51", None)
        self.k_52 = kw.get("k_52", None)
        self.k_53 = kw.get("k_53", None)
        self.k_54 = kw.get("k_54", None)
        self.k_55 = kw.get("k_55", None)
        self.k_56 = kw.get("k_56", None)
        self.k_57 = kw.get("k_57", None)
        self.k_58 = kw.get("k_58", None)
        self.k_59 = kw.get("k_59", None)
        self.k_6 = kw.get("k_6", None)
        self.k_60 = kw.get("k_60", None)
        self.k_61 = kw.get("k_61", None)
        self.k_62 = kw.get("k_62", None)
        self.k_63 = kw.get("k_63", None)
        self.k_64 = kw.get("k_64", None)
        self.k_65 = kw.get("k_65", None)
        self.k_66 = kw.get("k_66", None)
        self.k_67 = kw.get("k_67", None)
        self.k_68 = kw.get("k_68", None)
        self.k_69 = kw.get("k_69", None)
        self.k_7 = kw.get("k_7", None)
        self.k_70 = kw.get("k_70", None)
        self.k_71 = kw.get("k_71", None)
        self.k_72 = kw.get("k_72", None)
        self.k_73 = kw.get("k_73", None)
        self.k_74 = kw.get("k_74", None)
        self.k_75 = kw.get("k_75", None)
        self.k_76 = kw.get("k_76", None)
        self.k_77 = kw.get("k_77", None)
        self.k_78 = kw.get("k_78", None)
        self.k_79 = kw.get("k_79", None)
        self.k_8 = kw.get("k_8", None)
        self.k_80 = kw.get("k_80", None)
        self.k_81 = kw.get("k_81", None)
        self.k_82 = kw.get("k_82", None)
        self.k_83 = kw.get("k_83", None)
        self.k_84 = kw.get("k_84", None)
        self.k_85 = kw.get("k_85", None)
        self.k_86 = kw.get("k_86", None)
        self.k_87 = kw.get("k_87", None)
        self.k_88 = kw.get("k_88", None)
        self.k_89 = kw.get("k_89", None)
        self.k_9 = kw.get("k_9", None)
        self.k_90 = kw.get("k_90", None)
        self.k_91 = kw.get("k_91", None)
        self.k_92 = kw.get("k_92", None)
        self.k_93 = kw.get("k_93", None)
        self.k_94 = kw.get("k_94", None)
        self.k_95 = kw.get("k_95", None)
        self.k_96 = kw.get("k_96", None)
        self.k_97 = kw.get("k_97", None)
        self.k_98 = kw.get("k_98", None)
        self.k_99 = kw.get("k_99", None)
        self.metadata = kw.get("metadata", None)
        self.update_ts = kw.get("update_ts", None)
        self.year = kw.get("year", None)


    @staticmethod
    def updateConfig(id_, k, v, session):
        import time
        ts = int(time.time())
        sql = "UPDATE car_detail set " + str(k) + " = '" + str(v) + "',update_ts = " + str(ts) + " WHERE id = " + str(id_) + " LIMIT 1"
        res_info = session.execute(sql)
        return res_info


if __name__ == '__main__':
    import types

    print("""def __init__(self, *arg, **kw):""")
    for k in dir(CarDetailModel):
        if not (str(k).startswith("_") or isinstance(getattr(CarDetailModel, k), types.FunctionType)):
            print("""    self.{attr} = kw.get("{attr}", None)""".format(attr=k))



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2018/9/21 23:07
# 重新解析缓存文件，生成Item
import os
import pickle

import XX.File.FileHelper as uf


def reload_cache(cache_rp):
    for fp, fn in uf.FileHelper.get_file_list(cache_rp):
        yield pickle.load(open(fp + os.sep + fn, "rb"))



# -*- coding:utf-8 -*-
import time

import XX.Encrypt.EncryptHelper as cenc


# 服务端生成的response json结构封装
class CrawlerUrl(object):
    url = None
    url_md5 = None
    spider = None
    post_data = None
    url_type = None
    source_url_id = None
    cookie = None
    data_type = None
    resp_status = None
    retry_num = None
    has_next_page = None
    except_type = None
    except_msg = None
    crawled_item = None
    other = None
    id_del = None
    update_ts = None
    create_ts = None

    def __init__(self, *arg, **kw):
        self.url = kw.get("url")
        self.url_md5 = cenc.Encrypt.md5(self.url)
        self.spider = kw.get("spider")
        self.post_data = kw.get("post_data")
        self.url_type = kw.get("url_type")
        self.source_url_id = kw.get("source_url_id")
        self.cookie = kw.get("cookie")
        self.data_type = kw.get("data_type")
        self.resp_status = kw.get("resp_status")
        self.retry_num = kw.get("retry_num")
        self.has_next_page = kw.get("has_next_page")
        self.except_type = kw.get("except_type")
        self.except_msg = kw.get("except_msg")
        self.crawled_item = kw.get("crawled_item")
        self.other = kw.get("other")
        self.id_del = kw.get("id_del")
        self.update_ts = kw.get("update_ts")
        self.create_ts = kw.get("create_ts")

    @staticmethod
    def save_2_mysql(self, data):
        ts = int(time.time())
        val = data.get("url"), data.get("url_md5"), data.get("post_data"), data.get("url_type"), data.get(
            "source_url_id"), data.get("cookie"), data.get("data_type"), data.get("resp_status"), data.get(
            "retry_num"), data.get("has_next_page"), data.get("id_del"), data.get("except_type"), data.get(
            "except_msg"), 0, data.get("update_ts", ), data.get("create_ts"), ts, ts
        sql = "INSERT INTO data values(null,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)"
        return sql, val


if __name__ == "__main__":
    res = CrawlerUrl(url_type=0, url="baidu.com")
    import json

    print(json.dumps(res.__dict__))
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time         : 2019/2/22 16:17
import XX.HTML.HtmlHelper as UH


class DataSheetCookie(object):

    def process_request(self, request, spider):
        # cookie = "IPLOC=CN1200; SUID=3060247D2013940A000000005C2EBD27; SUV=1546566951191080; pgv_pvi=2621366272; pgv_si=s5189274624; ABTEST=8|1546566953|v1; SNUID={0}; weixinIndexVisited=1"
        cookie = "CFID=Z4iqyj5ekgtiqnnaafjwnjfv918npvjndfx6r17xcvr0lrdc1ny-238338775; CFTOKEN=Z4iqyj5ekgtiqnnaafjwnjfv918npvjndfx6r17xcvr0lrdc1ny-65854503; visid_incap_1764171=xbqVUiZjTFKxsabma8pwe4ZcalwAAAAAQUIPAAAAAABVRMI0i5CoEGeGfS/SYej4; s_fid=4EEE4CEEAC95F2ED-26088EDF3300658E; __gads=ID=11e416ab98d3cf23:T=1550474464:S=ALNI_MbQJ38X36gKJd6r3xoKDTIn2VSWqg; s_nr=1550478519334; incap_ses_560_1764171=Pp2RNCOt7hTsx/YpKYbFB/itb1wAAAAAZGUo2eH38qr6e/X0HSdRwg==; OptanonConsent=landingPath=NotLandingPage&datestamp=Fri+Feb+22+2019+16%3A08%3A27+GMT%2B0800+(%E4%B8%AD%E5%9B%BD%E6%A0%87%E5%87%86%E6%97%B6%E9%97%B4)&version=4.1.0&groups=1%3A1%2C2%3A0%2C3%3A0%2C4%3A0%2C0_20504%3A0%2C0_20505%3A0&AwaitingReconsent=false; s_cc=true; s_vnum=1553066448448%26vn%3D5; s_invisit=true; s_lv=1550822907500; s_lv_s=Less%20than%201%20day; s_sq=%5B%5BB%5D%5D; __atuvc=58%7C8; __atuvs=5c6fadfb49f4aa32000; __utma=1.736096348.1550474452.1550807780.1550822909.6; __utmc=1; __utmz=1.1550822909.6.4.utmcsr=baidu|utmccn=(organic)|utmcmd=organic; __utmt=1; __utmb=1.1.10.1550822909"
        cookie = UH.get_cookie_from_str(cookie)
        request.cookies = cookie
        request.headers.setdefault('Referer', "https://www.datasheets360.com/part/detail/5962-8870101xx/2044037045222141529/")
        print("Add Cookie---")



# -*- coding: utf-8 -*-

def get_hbase_conn_cfg(**kw):
    d = dict()
    d["host"] = kw.get("host", "localhost")
    # d["username"] = kw.get("username", None)
    # d["password"] = kw.get("pwd", None)
    d["port"] = kw.get("port", 9090)
    d["table"] = kw.get("table", "default")
    return d

# ubuntu_cfg = functools.partial(get_hbase_conn_cfg, host="192.168.1.44")
# zhihan00_cfg = functools.partial(get_hbase_conn_cfg, host="192.168.1.50")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/8/16 16:39
# @Author   : Peter
# @Des       : 
# @File        : ProductForum
# @Software: PyCharm
from sqlalchemy import Column, Integer, Index, String, Text
from sqlalchemy.ext.declarative import declarative_base

from XX.Model.SqlAlchemy.BaseModel import BaseModel

Base = declarative_base()
metadata = Base.metadata


class ProductForum(Base, BaseModel):
    __tablename__ = 'product_forum'
    __table_args__ = (
        Index('wy', 'product_id', 'forum_id', unique=True),
    )

    id = Column(Integer, primary_key=True)
    product_id = Column(Integer, nullable=False)
    forum_id = Column(Integer, nullable=False)
    url = Column(String(512), nullable=False)
    forum_name = Column(String(512), nullable=False)
    product_name = Column(String(512), nullable=False)
    forum_car_name = Column(String(512), nullable=False)
    is_del = Column(Integer)
    create_ts = Column(Integer)
    update_ts = Column(Integer)

    def __init__(self, *arg, **kw):
        self.forum_id = kw.get("forum_id", None)
        self.id = kw.get("id", None)
        self.meta_data = kw.get("meta_data", None)
        self.product_id = kw.get("product_id", None)
        self.product_name = kw.get("product_name", None)
        self.forum_name = kw.get("forum_name", None)
        self.forum_car_name = kw.get("forum_car_name", None)
        self.url = kw.get("url", None)
        self.update_ts = kw.get("update_ts", None)
        self.is_del = kw.get("is_del", None)
        self.create_ts = kw.get("create_ts", None)

    @staticmethod
    def getForumUrlsByForumId(forum_id, session):
        return session.query(ProductForum).filter(ProductForum.forum_id == forum_id).all()

    @staticmethod
    def getForumUrlByProductId(product_id, session):
        return session.query(ProductForum).filter(ProductForum.product_id == product_id).all()

    @staticmethod
    def getForumUrlByProductAndForum(product_id, forum_id, session):
        return session.query(ProductForum).filter(ProductForum.product_id == product_id, ProductForum.forum_id == forum_id).all()



# -*- coding: utf-8 -*-
import decimal
from datetime import datetime, date

import XX.Dict.DictHelper


class ListHelper:
    @staticmethod
    def decode_v(d, coding="utf-8"):
        ll = []
        for v in d:
            if type(v) == list or type(v) == tuple:
                ll.append(ListHelper.decode_v(list(v), coding))
            elif type(v) == bytes:
                ll.append(v.decode(coding))
            elif type(v) == dict:
                ll.append(XX.Dict.DictHelper.DictHelper.decode_v(v, coding))
            elif isinstance(v, decimal.Decimal):
                ll.append(float(v))
            elif isinstance(v, datetime):
                ll.append(v.strftime('%Y-%m-%d %H:%M:%S'))
            elif isinstance(v, date):
                ll.append(v.strftime('%Y-%m-%d'))
            else:
                ll.append(v)
        return ll


def count_dict_list_by_k_v(dict_list, unique_key="k", count_key="s"):
    word_count = {}
    for one in dict_list:
        word_count[one[unique_key]] = 1 + word_count.get(one[unique_key], 0)
    r = []
    for one in dict_list:
        if word_count.get(one[unique_key]):
            one[count_key] = word_count[one[unique_key]]
            del word_count[one[unique_key]]
            r.append(one)
    return r


if __name__ == '__main__':
    s = [{'web_id': b'a', 'id': 4}, {'web_id': b'a0', 'id': 3}, {'web_id': b'a00', 'id': 2}, {'web_id': b'b', 'id': 5},
         {'web_id': b'c', 'id': 6}, {'web_id': b'd', 'id': 7}, {'web_id': b'mb', 'id': 12}, {'web_id': b'mpv', 'id': 9},
         {'web_id': b'p', 'id': 11}, {'web_id': b'qk', 'id': 13}, {'web_id': b's', 'id': 10},
         {'web_id': b'suva', 'id': 16}, {'web_id': b'suva0', 'id': 15}, {'web_id': b'suvb', 'id': 17},
         {'web_id': b'suvc', 'id': 18},
         {'web_id': b'suvd', 'id': 19}]
    t = ListHelper.decode_v(s)
    print(t)

    pass



# -*- coding:utf-8 -*-
import time


class NewsItem:
    # 新闻发布时间
    datetime = ""
    # 更多模式
    more_mode = ""
    # create_time
    create_time = int(time.time())
    # 是否有相册
    has_gallery = False
    # 网站id
    id = ""
    # 用户id
    user_id = ""
    # 标题
    title = ""
    # 是否有视频
    has_video = False
    # 视频地址
    videos = []
    # 分享url
    share_url = ""
    # 源链接
    source = ""
    # 评论总数
    comment_count = 0
    # 文章链接
    article_url = ""
    # 评论总数
    comments_count = 0
    # 大模式?
    large_mode = ""
    # 摘要
    abstract = ""
    # 媒体链接
    media_url = ""
    # 媒体头像链接
    media_avatar_url = ""
    # 中等模式
    middle_mode = ""
    # 媒体创建者ID
    media_creator_id = ""
    # 标签id
    tag_id = ""
    # 来源链接
    source_url = ""
    # item id
    item_id = ""
    # 用户信息
    user_auth_info = {}
    # seo链接
    seo_url = ""
    # 关键词
    keyword = ""
    # 成为热点时间
    behot_time = ""
    # 标签
    tag = ""
    # 图片url
    image_url = []
    # 是否有图片
    has_image = False
    # highlight推荐？强调
    highlight = {}
    # 组id
    group_id = ""
    # 图片数量
    image_count = 0
    # 翻页页码
    page_index = 1
    # 内容
    content = ""

    def __init__(self, *arg, **kw):
        self.datetime = kw.pop("datetime", "")
        self.more_mode = kw.pop("more_mode", "")
        self.create_time = kw.pop("create_time", int(time.time()))
        self.has_gallery = kw.pop("has_gallery", False)
        self.id = kw.pop("id", "")
        self.user_id = kw.pop("user_id", "")
        self.title = kw.pop("title", "")
        self.has_video = kw.pop("has_video", False)
        self.videos = kw.pop("videos", [])
        self.share_url = kw.pop("share_url", "")
        self.source = kw.pop("source", "")
        self.comment_count = kw.pop("comment_count", 0)
        self.article_url = kw.pop("article_url", "")
        self.comments_count = kw.pop("comments_count", 0)
        self.large_mode = kw.pop("large_mode", "")
        self.abstract = kw.pop("abstract", "")
        self.media_url = kw.pop("media_url", "")
        self.media_avatar_url = kw.pop("media_avatar_url", "")
        self.middle_mode = kw.pop("middle_mode", "")
        self.media_creator_id = kw.pop("media_creator_id", "")
        self.tag_id = kw.pop("tag_id", "")
        self.source_url = kw.pop("source_url", "")
        self.item_id = kw.pop("item_id", "")
        self.user_auth_info = kw.pop("user_auth_info", {})
        self.seo_url = kw.pop("seo_url", "")
        self.keyword = kw.pop("keyword", "")
        self.behot_time = kw.pop("behot_time", "")
        self.tag = kw.pop("tag", "")
        self.image_url = kw.pop("image_url", [])
        self.has_image = kw.pop("has_image", False)
        self.highlight = kw.pop("highlight", {})
        self.group_id = kw.pop("group_id", "")
        self.image_count = kw.pop("image_count", 0)
        self.page_index = kw.pop("page_index", 1)
        self.content = kw.pop("content", "")


one_data = {
    "datetime": "2018-05-17 03:27:22",
    "more_mode": "false",
    "create_time": "1526498842",
    "has_gallery": "false",
    "id": "6556262604904333838",
    "user_id": 3656017252,
    "title": "联想高层再回应争议的“5G标准投票”事件 称是恶毒攻击",
    "has_video": "false",
    "share_url": "http://rss.cngold.com.cn/Articles/ArtContent/1/20180517d1702n237474655.html",
    "source": "中金网",
    "comment_count": 233,
    "article_url": "http://rss.cngold.com.cn/Articles/ArtContent/1/20180517d1702n237474655.html",
    "comments_count": 233,
    "large_mode": "false",
    "abstract": "5月16日，联想副总裁黄莹称，如果外部力量和不实信息干扰破坏了中国5G企业团结，很可能打破目前中美欧三方力量平衡，导致中国企业在5G标准制定中处于极为不利的位置。",
    "media_url": "http://toutiao.com/m3659626051/",
    "media_avatar_url": "//p1.pstatp.com/medium/1541/4152355309",
    "middle_mode": "false",
    "media_creator_id": 3656017252,
    "tag_id": 6556262604904333838,
    "source_url": "/group/6556262604904333838/",
    "item_id": "6556262604904333838",
    "user_auth_info": {
        "auth_type": "0",
        "other_auth": {
            "pgc": "头条财经作者"
        },
        "auth_info": "头条财经作者"
    },
    "seo_url": "/group/6556262604904333838/",
    "keyword": "联想",
    "behot_time": "1526498842",
    "tag": "news",
    "image_url": "//p3.pstatp.com/list/pgc-image/1526498841878aea1c5d74a",
    "has_image": "true",
    "highlight": {
        "source": [],
        "abstract": [
            [6, 2]
        ],
        "title": [
            [0, 2]
        ]
    },
    "group_id": "6556262604904333838",
    "image_count": 1}

if __name__ == "__main__":
    news = NewsItem()
    print(news)
    print(news.__dict__)



# -*- coding:utf-8 -*-
from scrapy.http import Response as ScrapyResponse


# 请求服务端，返回的response
class ResponseObj(ScrapyResponse):
    url = ""
    status = None
    status_code = None
    # 默认是返回header
    headers = {}
    ResponseHeaders = {}
    RequestHeaders = {}
    content = None
    text = ""
    method = None
    msg = ""
    cache = 0
    cache_file = None
    data = {}
    cookies = {}
    encoding = "UTF-8"
    error_callback = None
    callback = None

    def __init__(self, *arg, **kw):
        super().__init__(self.url, *arg, **kw)
        for k, v in kw.items():
            setattr(self, k, v)
        self.url = kw.get("url")
        self.status = kw.get("status")
        self.status_code = kw.get("status_code")
        self.headers = kw.get("headers")
        self.ResponseHeaders = kw.get("ResponseHeaders")
        self.RequestHeaders = kw.get("RequestHeaders")
        self.content = kw.get("content")
        self.text = kw.get("text")
        self.method = kw.get("method")
        self.msg = kw.get("msg")
        self.cache = kw.get("cache")
        self.cache_file = kw.get("cache_file")
        self.data = kw.get("data")
        self.cookies = kw.get("cookies")
        self.encoding = kw.get("encoding")
        self.error_callback = kw.get("error_callback")
        self.callback = kw.get("callback")



if __name__ == "__main__":
    a = ResponseObj(a=1)
    print(a.a)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/12/19 14:40
# @Email     : billsteve@126.com
# @Des       : 
# @File        : Jsons
# @Software: PyCharm
from XX.Model.SqlAlchemy.BaseModel import *
from sqlalchemy import Column, Integer, String
from sqlalchemy.dialects.mysql.types import TINYINT
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class JsonModel(Base):
    __tablename__ = 'jsons'

    id = Column(Integer, primary_key=True)
    jsons = Column(String(255))

    def __init__(self, *arg, **kw):
        self.id = kw.get("id", None)
        self.jsons = kw.get("jsons", None)
        self.metadata = kw.get("metadata", None)


if __name__ == '__main__':
    createInitFunction(JsonModel)



# -*- coding: utf-8 -*-
from sqlalchemy import Column, String
from sqlalchemy.dialects.mysql import INTEGER
from sqlalchemy.ext.declarative import declarative_base
from XX.Model.SqlAlchemy.BaseModel import *

Base = declarative_base()
metadata = Base.metadata


class WeiboComment(Base, BaseModel):
    __tablename__ = 'weibo_comment'

    id = Column(INTEGER(11), primary_key=True)
    weibo_id = Column(INTEGER(11))
    web_id =  Column(String(255))
    user_id = Column(INTEGER(11))
    p_id = Column(INTEGER(11))
    comments_num = Column(INTEGER(11))
    created_at = Column(String(255))
    rootid = Column(String(255))
    floor_number = Column(INTEGER(11))
    text = Column(String(255))
    disable_reply = Column(INTEGER(11))
    mid = Column(String(255))
    max_id = Column(INTEGER(11))
    total_number = Column(INTEGER(11))
    isLikedByMblogAuthor = Column(String(255))
    more_info_type = Column(INTEGER(11))
    bid = Column(String(255))
    source = Column(String(255))
    like_count = Column(INTEGER(11))
    liked = Column(String(255))
    is_del = Column(INTEGER(11))
    create_ts = Column(INTEGER(11))
    update_ts = Column(INTEGER(11))

    def __init__(self, *arg, **kw):
        self.bid = kw.get("bid", None)
        self.comments_num = kw.get("comments_num", None)
        self.create_ts = kw.get("create_ts", None)
        self.created_at = kw.get("created_at", None)
        self.disable_reply = kw.get("disable_reply", None)
        self.floor_number = kw.get("floor_number", None)
        self.id = kw.get("id", None)
        self.isLikedByMblogAuthor = kw.get("isLikedByMblogAuthor", None)
        self.is_del = kw.get("is_del", None)
        self.like_count = kw.get("like_count", None)
        self.liked = kw.get("liked", None)
        self.max_id = kw.get("max_id", None)
        self.metadata = kw.get("metadata", None)
        self.mid = kw.get("mid", None)
        self.more_info_type = kw.get("more_info_type", None)
        self.p_id = kw.get("p_id", None)
        self.rootid = kw.get("rootid", None)
        self.source = kw.get("source", None)
        self.text = kw.get("text", None)
        self.total_number = kw.get("total_number", None)
        self.update_ts = kw.get("update_ts", None)
        self.user_id = kw.get("user_id", None)
        self.web_id = kw.get("web_id", None)
        self.weibo_id = kw.get("weibo_id", None)


if __name__ == '__main__':
    createInitFunction(WeiboComment)



# -*- coding:utf-8 -*-
from setuptools import setup, find_packages
"""
rm -rf dist/* && python setup.py sdist_wheel --universal && twine upload dist/* -u billsteve -p aaasss25 --verbose
"""

setup(
    name="XX",
    version="3.3.0",
    description=(
        "Python tools for myself(billsteve@126.com)"
    ),
    long_description="""
    ## Just Test, Just for myself! Not safe,not stable and will boom boom boom. ##
    
    Just like these:
        -  https://music.163.com/song?id=3949444&userid=319278554  
        -  https://music.163.com/song?id=474581010&userid=319278554  
        -  https://open.spotify.com/album/47wyCwrChF50ZTFNOuWx99  
        -  https://open.spotify.com/track/3oDFtOhcN08qeDPAK6MEQG  
    
    Changelog:
        ### 3.2.4
            -  增加base64编码和解码
        ### 3.1.0
            -  Scrapy->DM->CacheDM::CacheFileRequest, del response.certificate.
        ### 3.0.6
            -  add get_cache_file in Scrapy->DM->CacheDM::CacheFileRequest.
                Could cache file by CacheFileRequest if you don't set FUN_CACHE_FILE_PATH in setting.               
        ### 3.0.4
            - add get p domain    
    """,
    author='bill steve',
    author_email='billsteve@126.com',
    maintainer='bill steve',
    maintainer_email='billsteve@126.com',
    license='MIT License',
    packages=find_packages(),
    platforms=["all"],
    url='https://github.com/billsteve',

    install_requires=[
        "scrapy",
        "scrapyd",
        "scrapyd-client",
        "scrapy-redis",
        "pyquery",
        "redis",
        "requests",
        "pymysql",
        "sqlalchemy",
        "logzero",
        "happybase",
        "tld"
    ],

    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
)



# -*- coding: utf-8 -*-

import XX.Date.DatetimeHelper as Dt
import XX.File.FileHelper as Fh


# 记录开始抓取的URL
class ToCrawlUrl(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        return cls()

    def process_request(self, request, spider):
        save_path = self.settings.get("ROOT_PATH_LOG") + Dt.get_today() + "_to_crawl.log"
        Fh.FileHelper.save_file(save_path, str(Dt.get_now_time()) + "\t" + spider.name + "\t" + request.url + "\n", method="a")


# 记录成功抓了的URL
class CrawledUrl(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        return cls()

    def process_response(self, request, response, spider):
        save_path = self.settings.get("ROOT_PATH_LOG") + Dt.get_today() + "_carwled.log"
        Fh.FileHelper.save_file(
            save_path,
            str(Dt.get_now_time()) + "\t" + spider.name + "\t" + str(response.status) + "\t" + request.url + "\n",
            method="a"
        )
        return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/12/12 16:15
# @Email     : billsteve@126.com
# @Des       : BossDetail
# @File        : BossDetail
# @Software: PyCharm
from XX.Model.SqlAlchemy.BaseModel import *
from sqlalchemy import Column, String
from sqlalchemy.dialects.mysql import INTEGER
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata


class BossDetail(Base, BaseModel):
    __tablename__ = 'boss_detail'

    id = Column(INTEGER(11), primary_key=True)
    boss_web_id = Column(INTEGER(11))
    name = Column(String(255))
    personid = Column(String(255))
    companyname = Column(String(255))
    companykey = Column(String(255))
    face_oss = Column(String(255))
    face = Column(String(255))
    link = Column(String(255))
    job = Column(String(255))
    weibo = Column(String(255))
    des = Column(String(255))
    country = Column(String(255))
    city = Column(String(255))
    county = Column(String(255))
    role = Column(String(255))
    edu = Column(String(255))
    everjob = Column(String(255))
    weixin = Column(String(255))
    phone = Column(String(255))
    email = Column(String(255))
    birth = Column(String(255))
    jiguan = Column(String(255))
    sex = Column(String(255))
    hangye = Column(String(255))
    biye = Column(String(255))
    xueli = Column(String(255))
    zhuanye = Column(String(255))
    minzu = Column(String(255))
    hobby = Column(String(255))
    milestone = Column(String(255))
    sayings = Column(String(255))
    type = Column(String(255))
    cleaning_type = Column(String(255))
    click_count = Column(String(255))
    status = Column(String(255))
    is_delete = Column(String(255))
    oper_name = Column(String(255))
    relativeInfoCount = Column(INTEGER(11))
    is_del = Column(INTEGER(11))
    create_ts = Column(INTEGER(11))
    update_ts = Column(INTEGER(11))

    def __init__(self, *arg, **kw):
        self.birth = kw.get("birth", None)
        self.biye = kw.get("biye", None)
        self.boss_web_id = kw.get("boss_web_id", None)
        self.city = kw.get("city", None)
        self.cleaning_type = kw.get("cleaning_type", None)
        self.click_count = kw.get("click_count", None)
        self.companykey = kw.get("companykey", None)
        self.companyname = kw.get("companyname", None)
        self.country = kw.get("country", None)
        self.county = kw.get("county", None)
        self.create_ts = kw.get("create_ts", None)
        self.des = kw.get("des", None)
        self.edu = kw.get("edu", None)
        self.email = kw.get("email", None)
        self.everjob = kw.get("everjob", None)
        self.face = kw.get("face", None)
        self.face_oss = kw.get("face_oss", None)
        self.hangye = kw.get("hangye", None)
        self.hobby = kw.get("hobby", None)
        self.id = kw.get("id", None)
        self.is_del = kw.get("is_del", None)
        self.is_delete = kw.get("is_delete", None)
        self.jiguan = kw.get("jiguan", None)
        self.job = kw.get("job", None)
        self.link = kw.get("link", None)
        self.metadata = kw.get("metadata", None)
        self.milestone = kw.get("milestone", None)
        self.minzu = kw.get("minzu", None)
        self.name = kw.get("name", None)
        self.oper_name = kw.get("oper_name", None)
        self.personid = kw.get("personid", None)
        self.phone = kw.get("phone", None)
        self.relativeInfoCount = kw.get("relativeInfoCount", None)
        self.role = kw.get("role", None)
        self.sayings = kw.get("sayings", None)
        self.sex = kw.get("sex", None)
        self.status = kw.get("status", None)
        self.type = kw.get("type", None)
        self.update_ts = kw.get("update_ts", None)
        self.weibo = kw.get("weibo", None)
        self.weixin = kw.get("weixin", None)
        self.xueli = kw.get("xueli", None)
        self.zhuanye = kw.get("zhuanye", None)


if __name__ == '__main__':
    createInitFunction(BossDetail)
